@@ -1,5 +1,3 @@
-from __future__ import absolute_import, division, print_function, unicode_literals
-
 ''' CHM File decoding support '''
 __license__ = 'GPL v3'
 __copyright__  = '2008, Kovid Goyal <kovid at kovidgoyal.net>,' \
@@ -10,14 +8,14 @@ import os
 from calibre.customize.conversion import InputFormatPlugin
 from calibre.ptempfile import TemporaryDirectory
 from calibre.constants import filesystem_encoding
-from polyglot.builtins import unicode_type
+from polyglot.builtins import as_bytes
 
 
 class CHMInput(InputFormatPlugin):
 
     name        = 'CHM Input'
     author      = 'Kovid Goyal and Alex Bramley'
-    description = 'Convert CHM files to OEB'
+    description = _('Convert CHM files to OEB')
     file_types  = {'chm'}
     commit_name = 'chm_input'
 
@@ -37,7 +35,7 @@ class CHMInput(InputFormatPlugin):
 
         log.debug('Processing CHM...')
         with TemporaryDirectory('_chm2oeb') as tdir:
-            if not isinstance(tdir, unicode_type):
+            if not isinstance(tdir, str):
                 tdir = tdir.decode(filesystem_encoding)
             html_input = plugin_for_input_format('html')
             for opt in html_input.options:
@@ -65,7 +63,6 @@ class CHMInput(InputFormatPlugin):
                 from calibre.ebooks.metadata.book.base import Metadata
                 metadata = Metadata(os.path.basename(chm_name))
             encoding = self._chm_reader.get_encoding() or options.input_encoding or 'cp1252'
-            self._chm_reader.CloseCHM()
             # print((tdir, mainpath))
             # from calibre import ipython
             # ipython()
@@ -76,6 +73,7 @@ class CHMInput(InputFormatPlugin):
             if os.path.abspath(mainpath) in self._chm_reader.re_encoded_files:
                 uenc = 'utf-8'
             htmlpath, toc = self._create_html_root(mainpath, log, uenc)
+            self._chm_reader.CloseCHM()
             oeb = self._create_oebbook_html(htmlpath, tdir, options, log, metadata)
             options.debug_pipeline = odi
             if toc.count() > 1:
@@ -104,7 +102,11 @@ class CHMInput(InputFormatPlugin):
         # use HTMLInput plugin to generate book
         from calibre.customize.builtins import HTMLInput
         opts.breadth_first = True
+        opts.max_levels = 30
+        opts.correct_case_mismatches = True
         htmlinput = HTMLInput(None)
+        htmlinput.set_root_dir_of_input(basedir)
+        htmlinput.root_dir_for_absolute_links = basedir
         oeb = htmlinput.create_oebbook(htmlpath, basedir, opts, log, mi)
         return oeb
 
@@ -113,7 +115,12 @@ class CHMInput(InputFormatPlugin):
         from polyglot.urllib import unquote as _unquote
         from calibre.ebooks.oeb.base import urlquote
         from calibre.ebooks.chardet import xml_to_unicode
-        hhcdata = self._read_file(hhcpath)
+        try:
+            hhcdata = self._read_file(hhcpath)
+        except FileNotFoundError:
+            log.warn('No HHC file found in CHM, using the default topic as the first HTML file')
+            from calibre.ebooks.oeb.base import TOC
+            return os.path.join(os.path.dirname(hhcpath), self._chm_reader.relpath_to_first_html_file()), TOC()
         hhcdata = hhcdata.decode(encoding)
         hhcdata = xml_to_unicode(hhcdata, verbose=True,
                             strip_encoding_pats=True, resolve_entities=True)[0]
@@ -128,22 +135,26 @@ class CHMInput(InputFormatPlugin):
         base = os.path.dirname(os.path.abspath(htmlpath))
 
         def unquote(x):
-            if isinstance(x, unicode_type):
+            if isinstance(x, str):
                 x = x.encode('utf-8')
             return _unquote(x).decode('utf-8')
 
         def unquote_path(x):
-            y = unquote(x)
-            if (not os.path.exists(os.path.join(base, x)) and os.path.exists(os.path.join(base, y))):
-                x = y
-            return x
+            x, _, frag = x.partition('#')
+            if frag:
+                frag = '#' + frag
+            if not os.path.exists(os.path.join(base, x)):
+                y = unquote(x)
+                if os.path.exists(os.path.join(base, y)):
+                    x = y
+            return x, frag
 
         def donode(item, parent, base, subpath):
             for child in item:
                 title = child.title
                 if not title:
                     continue
-                raw = unquote_path(child.href or '')
+                raw, frag = unquote_path(child.href or '')
                 rsrcname = os.path.basename(raw)
                 rsrcpath = os.path.join(subpath, rsrcname)
                 if (not os.path.exists(os.path.join(base, rsrcpath)) and os.path.exists(os.path.join(base, raw))):
@@ -153,7 +164,7 @@ class CHMInput(InputFormatPlugin):
                     rsrcpath = urlquote(rsrcpath)
                 if not raw:
                     rsrcpath = ''
-                c = DIV(A(title, href=rsrcpath))
+                c = DIV(A(title, href=rsrcpath + frag))
                 donode(child, c, base, subpath)
                 parent.append(c)
 
@@ -161,7 +172,7 @@ class CHMInput(InputFormatPlugin):
             if toc.count() > 1:
                 from lxml.html.builder import HTML, BODY, DIV, A
                 path0 = toc[0].href
-                path0 = unquote_path(path0)
+                path0 = unquote_path(path0)[0]
                 subpath = os.path.dirname(path0)
                 base = os.path.dirname(f.name)
                 root = DIV()
@@ -170,17 +181,17 @@ class CHMInput(InputFormatPlugin):
                                    pretty_print=True)
                 f.write(raw)
             else:
-                f.write(hhcdata)
+                f.write(as_bytes(hhcdata))
         return htmlpath, toc
 
     def _read_file(self, name):
-        with lopen(name, 'rb') as f:
+        with open(name, 'rb') as f:
             data = f.read()
         return data
 
     def add_node(self, node, toc, ancestor_map):
         from calibre.ebooks.chm.reader import match_string
-        if match_string(node.attrib['type'], 'text/sitemap'):
+        if match_string(node.attrib.get('type', ''), 'text/sitemap'):
             p = node.xpath('ancestor::ul[1]/ancestor::li[1]/object[1]')
             parent = p[0] if p else None
             toc = ancestor_map.get(parent, toc)
