@@ -1,26 +1,34 @@
-#!/usr/bin/env python2
-# vim:fileencoding=UTF-8:ts=4:sw=4:sta:et:sts=4:ai
-from __future__ import (unicode_literals, division, absolute_import,
-                        print_function)
+#!/usr/bin/env python
+
 
 __license__   = 'GPL v3'
 __copyright__ = '2012, Kovid Goyal <kovid@kovidgoyal.net>'
 __docformat__ = 'restructuredtext en'
 
-import re, tempfile, os
+import os
+import re
+import tempfile
+from contextlib import suppress
 from functools import partial
-from itertools import izip
+from urllib.parse import quote
 
-from calibre.constants import islinux, isbsd
-from calibre.customize.conversion import (InputFormatPlugin,
-        OptionRecommendation)
-from calibre.utils.localization import get_lang
-from calibre.utils.filenames import ascii_filename
+from calibre.constants import filesystem_encoding, isbsd, islinux
+from calibre.customize.conversion import InputFormatPlugin, OptionRecommendation
+from calibre.utils.filenames import (
+    ascii_filename, case_ignoring_open_file, get_long_path_name,
+)
 from calibre.utils.imghdr import what
+from calibre.utils.localization import __, get_lang
+from polyglot.builtins import as_unicode
 
 
 def sanitize_file_name(x):
-    ans = re.sub(r'\s+', ' ', re.sub(r'[?&=;#]', '_', ascii_filename(x))).strip().rstrip('.')
+    ans = re.sub(r'\s+', ' ', ascii_filename(x))
+    for ch in '?&=;#/\\':
+        ans = ans.replace(ch, '_')
+        q = quote(ch, safe='')
+        ans = re.sub(f'\\{q}', '_', ans, flags=re.I)
+    ans = ans.strip().rstrip('.')
     ans, ext = ans.rpartition('.')[::2]
     return (ans.strip() + '.' + ext.strip()).rstrip('.')
 
@@ -29,9 +37,10 @@ class HTMLInput(InputFormatPlugin):
 
     name        = 'HTML Input'
     author      = 'Kovid Goyal'
-    description = 'Convert HTML and OPF files to an OEB'
+    description = _('Convert HTML and OPF files to an OEB')
     file_types  = {'opf', 'html', 'htm', 'xhtml', 'xhtm', 'shtm', 'shtml'}
     commit_name = 'html_input'
+    root_dir_for_absolute_links = ''
 
     options = {
         OptionRecommendation(name='breadth_first',
@@ -59,18 +68,35 @@ class HTMLInput(InputFormatPlugin):
                 )
         ),
 
+        OptionRecommendation(name='allow_local_files_outside_root',
+            recommended_value=False, level=OptionRecommendation.LOW,
+            help=_('Normally, resources linked to by the HTML file or its children will only be allowed'
+                   ' if they are in a sub-folder of the original HTML file. This option allows including'
+                   ' local files from any location on your computer. This can be a security risk if you'
+                   ' are converting untrusted HTML and expecting to distribute the result of the conversion.'
+                )
+        ),
+
+
     }
 
+    def set_root_dir_of_input(self, basedir):
+        self.root_dir_of_input = os.path.normcase(get_long_path_name(os.path.abspath(basedir)) + os.sep)
+
     def convert(self, stream, opts, file_ext, log,
                 accelerators):
         self._is_case_sensitive = None
-        basedir = os.getcwdu()
+        basedir = os.getcwd()
         self.opts = opts
 
         fname = None
         if hasattr(stream, 'name'):
-            basedir = os.path.dirname(stream.name)
-            fname = os.path.basename(stream.name)
+            sname = stream.name
+            if isinstance(sname, bytes):
+                sname = sname.decode(filesystem_encoding)
+            basedir = os.path.dirname(sname)
+            fname = os.path.basename(sname)
+        self.set_root_dir_of_input(basedir)
 
         if file_ext != 'opf':
             if opts.dont_package:
@@ -98,19 +124,22 @@ class HTMLInput(InputFormatPlugin):
         return self._is_case_sensitive
 
     def create_oebbook(self, htmlpath, basedir, opts, log, mi):
+        import css_parser
+        import logging
         import uuid
-        from calibre.ebooks.conversion.plumber import create_oebbook
-        from calibre.ebooks.oeb.base import (DirContainer,
-            rewrite_links, urlnormalize, urldefrag, BINARY_MIME, OEB_STYLES,
-            xpath, urlquote)
+
         from calibre import guess_type
-        from calibre.ebooks.oeb.transforms.metadata import \
-            meta_info_to_oeb_metadata
+        from calibre.ebooks.conversion.plumber import create_oebbook
         from calibre.ebooks.html.input import get_filelist
         from calibre.ebooks.metadata import string_to_authors
+        from calibre.ebooks.oeb.base import (
+            BINARY_MIME, OEB_STYLES, DirContainer, rewrite_links, urldefrag,
+            urlnormalize, urlquote, xpath,
+        )
+        from calibre.ebooks.oeb.transforms.metadata import meta_info_to_oeb_metadata
         from calibre.utils.localization import canonicalize_lang
-        import cssutils, logging
-        cssutils.log.setLevel(logging.WARN)
+        self.opts = opts
+        css_parser.log.setLevel(logging.WARN)
         self.OEB_STYLES = OEB_STYLES
         oeb = create_oebbook(log, None, opts, self,
                 encoding=opts.input_encoding, populate=False)
@@ -121,7 +150,7 @@ class HTMLInput(InputFormatPlugin):
         if not metadata.language:
             l = canonicalize_lang(getattr(opts, 'language', None))
             if not l:
-                oeb.logger.warn(u'Language not specified')
+                oeb.logger.warn('Language not specified')
                 l = get_lang().replace('_', '-')
             metadata.add('language', l)
         if not metadata.creator:
@@ -169,6 +198,7 @@ class HTMLInput(InputFormatPlugin):
         self.urlnormalize, self.DirContainer = urlnormalize, DirContainer
         self.urldefrag = urldefrag
         self.guess_type, self.BINARY_MIME = guess_type, BINARY_MIME
+        self.stylesheets_to_process = []
 
         self.log('Rewriting HTML links')
         for f in filelist:
@@ -182,15 +212,14 @@ class HTMLInput(InputFormatPlugin):
                 item = oeb.manifest.hrefs[urlnormalize(href)]
             rewrite_links(item.data, partial(self.resource_adder, base=dpath))
 
-        for item in oeb.manifest.values():
+        while self.stylesheets_to_process:
+            sheet = self.stylesheets_to_process.pop()
+            css_parser.replaceUrls(sheet.data, partial(self.resource_adder, base=sheet.html_input_dirpath))
+        for item in oeb.manifest:
             if item.media_type in self.OEB_STYLES:
-                dpath = None
-                for path, href in self.added_resources.items():
-                    if href == item.href:
-                        dpath = os.path.dirname(path)
-                        break
-                cssutils.replaceUrls(item.data,
-                        partial(self.resource_adder, base=dpath))
+                item.resolve_css_imports = True
+                item.override_css_fetch = None
+                item.reparse_css()
 
         toc = self.oeb.toc
         self.oeb.auto_generated_toc = True
@@ -215,24 +244,27 @@ class HTMLInput(InputFormatPlugin):
         use = titles
         if len(titles) > len(set(titles)):
             use = headers
-        for title, item in izip(use, self.oeb.spine):
+        for title, item in zip(use, self.oeb.spine):
             if not item.linear:
                 continue
             toc.add(title, item.href)
 
-        oeb.container = DirContainer(os.getcwdu(), oeb.log, ignore_opf=True)
+        oeb.container = DirContainer(os.getcwd(), oeb.log, ignore_opf=True)
         return oeb
 
     def link_to_local_path(self, link_, base=None):
         from calibre.ebooks.html.input import Link
-        if not isinstance(link_, unicode):
+        if not isinstance(link_, str):
             try:
                 link_ = link_.decode('utf-8', 'error')
             except:
                 self.log.warn('Failed to decode link %r. Ignoring'%link_)
                 return None, None
+        if self.root_dir_for_absolute_links and link_.startswith('/'):
+            link_ = link_.lstrip('/')
+            base = self.root_dir_for_absolute_links
         try:
-            l = Link(link_, base if base else os.getcwdu())
+            l = Link(link_, base if base else os.getcwd())
         except:
             self.log.exception('Failed to process link: %r'%link_)
             return None, None
@@ -243,10 +275,17 @@ class HTMLInput(InputFormatPlugin):
         frag = l.fragment
         if not link:
             return None, None
+        link = os.path.abspath(os.path.realpath(link))
+        q = os.path.normcase(get_long_path_name(link))
+        if not q.startswith(self.root_dir_of_input):
+            if not self.opts.allow_local_files_outside_root:
+                if os.path.exists(q):
+                    self.log.warn(f'Not adding {q} as it is outside the document root: {self.root_dir_of_input}')
+                return None, None
         return link, frag
 
     def resource_adder(self, link_, base=None):
-        from urllib import quote
+        from polyglot.urllib import quote
         link, frag = self.link_to_local_path(link_, base=base)
         if link is None:
             return link_
@@ -257,17 +296,24 @@ class HTMLInput(InputFormatPlugin):
         except:
             return link_
         if not os.access(link, os.R_OK):
-            return link_
+            corrected = False
+            if getattr(self.opts, 'correct_case_mismatches', False):
+                with suppress(OSError), case_ignoring_open_file(link) as f:
+                    link = f.name
+                    corrected = True
+            if not corrected:
+                return link_
         if os.path.isdir(link):
             self.log.warn(link_, 'is a link to a directory. Ignoring.')
             return link_
         if not self.is_case_sensitive(tempfile.gettempdir()):
             link = link.lower()
         if link not in self.added_resources:
+            guessed = self.guess_type(os.path.basename(link))[0]
+            media_type = guessed or self.BINARY_MIME
+            is_stylesheet = media_type in self.OEB_STYLES
             bhref = os.path.basename(link)
             id, href = self.oeb.manifest.generate(id='added', href=sanitize_file_name(bhref))
-            guessed = self.guess_type(href)[0]
-            media_type = guessed or self.BINARY_MIME
             if media_type == 'text/plain':
                 self.log.warn('Ignoring link to text file %r'%link_)
                 return None
@@ -275,13 +321,13 @@ class HTMLInput(InputFormatPlugin):
                 # Check for the common case, images
                 try:
                     img = what(link)
-                except EnvironmentError:
+                except OSError:
                     pass
                 else:
                     if img:
                         media_type = self.guess_type('dummy.'+img)[0] or self.BINARY_MIME
 
-            self.oeb.log.debug('Added', link)
+            self.oeb.log.debug('Added', link, 'with href:', href)
             self.oeb.container = self.DirContainer(os.path.dirname(link),
                     self.oeb.log, ignore_opf=True)
             # Load into memory
@@ -289,12 +335,14 @@ class HTMLInput(InputFormatPlugin):
             # bhref refers to an already existing file. The read() method of
             # DirContainer will call unquote on it before trying to read the
             # file, therefore we quote it here.
-            if isinstance(bhref, unicode):
+            if isinstance(bhref, str):
                 bhref = bhref.encode('utf-8')
-            item.html_input_href = quote(bhref).decode('utf-8')
-            if guessed in self.OEB_STYLES:
-                item.override_css_fetch = partial(
-                        self.css_import_handler, os.path.dirname(link))
+            item.html_input_href = as_unicode(quote(bhref))
+            if is_stylesheet:
+                item.html_input_dirpath = os.path.dirname(link)
+                item.resolve_css_imports = False
+                item.override_css_fetch = lambda url: (None, '')
+                self.stylesheets_to_process.append(item)
             item.data
             self.added_resources[link] = href
 
@@ -302,15 +350,3 @@ class HTMLInput(InputFormatPlugin):
         if frag:
             nlink = '#'.join((nlink, frag))
         return nlink
-
-    def css_import_handler(self, base, href):
-        link, frag = self.link_to_local_path(href, base=base)
-        if link is None or not os.access(link, os.R_OK) or os.path.isdir(link):
-            return (None, None)
-        try:
-            raw = open(link, 'rb').read().decode('utf-8', 'replace')
-            raw = self.oeb.css_preprocessor(raw, add_namespace=False)
-        except:
-            self.log.exception('Failed to read CSS file: %r'%link)
-            return (None, None)
-        return (None, raw)
