@@ -1,6 +1,3 @@
-# -*- coding: utf-8 -*-
-
-
 __license__ = 'GPL 3'
 __copyright__ = '2009, Kovid Goyal <kovid@kovidgoyal.net>'
 __docformat__ = 'restructuredtext en'
@@ -143,7 +140,7 @@ OptionRecommendation(name='output_profile',
             choices=[x.short_name for x in output_profiles()],
             help=_('Specify the output profile. The output profile '
                    'tells the conversion system how to optimize the '
-                   'created document for the specified device (such as by resizing images for the device screen size). In some cases, '
+                   'created document for the specified device. In some cases, '
                    'an output profile can be used to optimize the output for a particular device, but this is rarely necessary. '
                    'Choices are:') + ', '.join([
                        x.short_name for x in output_profiles()])
@@ -369,6 +366,12 @@ OptionRecommendation(name='transform_css_rules',
                    ' rules are applied after all other CSS processing is done.')
         ),
 
+OptionRecommendation(name='transform_html_rules',
+            recommended_value=None, level=OptionRecommendation.LOW,
+            help=_('Rules for transforming the HTML in this book. These'
+                   ' rules are applied after the HTML is parsed, but before any other transformations.')
+        ),
+
 OptionRecommendation(name='filter_css',
             recommended_value=None, level=OptionRecommendation.LOW,
             help=_('A comma separated list of CSS properties that '
@@ -881,7 +884,7 @@ OptionRecommendation(name='search_replace',
             if name in {'sr1_search', 'sr1_replace', 'sr2_search', 'sr2_replace', 'sr3_search', 'sr3_replace', 'filter_css', 'comments'}:
                 if not a and not b:
                     return True
-            if name in {'transform_css_rules', 'search_replace'}:
+            if name in {'transform_css_rules', 'transform_html_rules', 'search_replace'}:
                 if b == '[]':
                     b = None
             return a == b
@@ -945,7 +948,7 @@ OptionRecommendation(name='search_replace',
         if self.opts.read_metadata_from_opf is not None:
             self.opts.read_metadata_from_opf = os.path.abspath(
                                             self.opts.read_metadata_from_opf)
-            with lopen(self.opts.read_metadata_from_opf, 'rb') as stream:
+            with open(self.opts.read_metadata_from_opf, 'rb') as stream:
                 opf = OPF(stream, os.path.dirname(self.opts.read_metadata_from_opf))
             mi = opf.to_book_metadata()
         self.opts_to_mi(mi)
@@ -955,7 +958,7 @@ OptionRecommendation(name='search_replace',
             ext = mi.cover.rpartition('.')[-1].lower().strip()
             if ext not in ('png', 'jpg', 'jpeg', 'gif'):
                 ext = 'jpg'
-            with lopen(mi.cover, 'rb') as stream:
+            with open(mi.cover, 'rb') as stream:
                 mi.cover_data = (ext, stream.read())
             mi.cover = None
         self.user_metadata = mi
@@ -1061,7 +1064,7 @@ OptionRecommendation(name='search_replace',
             self.opts.debug_pipeline = os.path.abspath(self.opts.debug_pipeline)
             if not os.path.exists(self.opts.debug_pipeline):
                 os.makedirs(self.opts.debug_pipeline)
-            with lopen(os.path.join(self.opts.debug_pipeline, 'README.txt'), 'wb') as f:
+            with open(os.path.join(self.opts.debug_pipeline, 'README.txt'), 'wb') as f:
                 f.write(DEBUG_README)
             for x in ('input', 'parsed', 'structure', 'processed'):
                 x = os.path.join(self.opts.debug_pipeline, x)
@@ -1079,7 +1082,7 @@ OptionRecommendation(name='search_replace',
 
         tdir = PersistentTemporaryDirectory('_plumber')
         stream = self.input if self.input_fmt == 'recipe' else \
-                lopen(self.input, 'rb')
+                open(self.input, 'rb')
         if self.input_fmt == 'recipe':
             self.opts.original_recipe_input_arg = self.original_input_arg
 
@@ -1133,6 +1136,13 @@ OptionRecommendation(name='search_replace',
 
         self.oeb.plumber_output_format = self.output_fmt or ''
 
+        if self.opts.transform_html_rules:
+            transform_html_rules = self.opts.transform_html_rules
+            if isinstance(transform_html_rules, string_or_bytes):
+                transform_html_rules = json.loads(transform_html_rules)
+            from calibre.ebooks.html_transform_rules import transform_conversion_book
+            transform_conversion_book(self.oeb, self.opts, transform_html_rules)
+
         from calibre.ebooks.oeb.transforms.data_url import DataURL
         DataURL()(self.oeb, self.opts)
         from calibre.ebooks.oeb.transforms.guide import Clean
@@ -1189,7 +1199,7 @@ OptionRecommendation(name='search_replace',
 
         if self.opts.extra_css and os.path.exists(self.opts.extra_css):
             with open(self.opts.extra_css, 'rb') as f:
-                self.opts.extra_css = f.read()
+                self.opts.extra_css = f.read().decode('utf-8')
 
         oibl = self.opts.insert_blank_line
         orps  = self.opts.remove_paragraph_spacing
