@@ -1,20 +1,23 @@
-#!/usr/bin/env python2
-# vim:fileencoding=UTF-8:ts=4:sw=4:sta:et:sts=4:ai
+#!/usr/bin/env python
+
 
 __license__   = 'GPL v3'
 __copyright__ = '2010, Kovid Goyal <kovid@kovidgoyal.net>'
 __docformat__ = 'restructuredtext en'
 
-import copy, traceback
+import copy
+import traceback
 
 from calibre import prints
-from calibre.constants import DEBUG, ispy3
-from calibre.ebooks.metadata.book import (SC_COPYABLE_FIELDS,
-        SC_FIELDS_COPY_NOT_NULL, STANDARD_METADATA_FIELDS,
-        TOP_LEVEL_IDENTIFIERS, ALL_METADATA_FIELDS)
+from calibre.constants import DEBUG
+from calibre.ebooks.metadata.book import (
+    ALL_METADATA_FIELDS, SC_COPYABLE_FIELDS, SC_FIELDS_COPY_NOT_NULL,
+    STANDARD_METADATA_FIELDS, TOP_LEVEL_IDENTIFIERS,
+)
 from calibre.library.field_metadata import FieldMetadata
-from calibre.utils.icu import sort_key
-from polyglot.builtins import iteritems, unicode_type, filter, map
+from calibre.utils.icu import lower as icu_lower, sort_key
+from calibre.utils.localization import ngettext
+from polyglot.builtins import iteritems, string_or_bytes
 
 # Special sets used to optimize the performance of getting and setting
 # attributes on Metadata objects
@@ -24,7 +27,10 @@ SIMPLE_SET = frozenset(SIMPLE_GET - {'identifiers'})
 
 def human_readable(size, precision=2):
     """ Convert a size in bytes into megabytes """
-    return ('%.'+str(precision)+'f'+ 'MB') % ((size/(1024.*1024.)),)
+    ans = size/(1024*1024)
+    if ans < 0.1:
+        return '<0.1 MB'
+    return ('%.'+str(precision)+'f'+ ' MB') % ans
 
 
 NULL_VALUES = {
@@ -39,7 +45,7 @@ NULL_VALUES = {
                 'author_sort'  : _('Unknown'),
                 'title'        : _('Unknown'),
                 'user_categories' : {},
-                'author_link_map' : {},
+                'link_maps'    : {},
                 'language'     : 'und'
 }
 
@@ -51,11 +57,15 @@ def reset_field_metadata():
     field_metadata = FieldMetadata()
 
 
-ck = lambda typ: icu_lower(typ).strip().replace(':', '').replace(',', '')
-cv = lambda val: val.strip().replace(',', '|')
+def ck(typ):
+    return icu_lower(typ).strip().replace(':', '').replace(',', '')
 
 
-class Metadata(object):
+def cv(val):
+    return val.strip().replace(',', '|')
+
+
+class Metadata:
 
     '''
     A class representing all the metadata for a book. The various standard metadata
@@ -140,9 +150,7 @@ class Metadata(object):
         if field in _data['user_metadata']:
             d = _data['user_metadata'][field]
             val = d['#value#']
-            if d['datatype'] != 'composite':
-                return val
-            if val is None:
+            if val is None and d['datatype'] == 'composite':
                 d['#value#'] = 'RECURSIVE_COMPOSITE FIELD (Metadata) ' + field
                 val = d['#value#'] = self.formatter.safe_format(
                                             d['display']['composite_template'],
@@ -181,8 +189,9 @@ class Metadata(object):
                 langs = [val]
             _data['languages'] = langs
         elif field in _data['user_metadata']:
-            _data['user_metadata'][field]['#value#'] = val
-            _data['user_metadata'][field]['#extra#'] = extra
+            d = _data['user_metadata'][field]
+            d['#value#'] = val
+            d['#extra#'] = extra
         else:
             # You are allowed to stick arbitrary attributes onto this object as
             # long as they don't conflict with global or user metadata names
@@ -195,11 +204,24 @@ class Metadata(object):
     def has_key(self, key):
         return key in object.__getattribute__(self, '_data')
 
+    def _evaluate_all_composites(self):
+        custom_fields = object.__getattribute__(self, '_data')['user_metadata']
+        for field in custom_fields:
+            self._evaluate_composite(field)
+
+    def _evaluate_composite(self, field):
+        f = object.__getattribute__(self, '_data')['user_metadata'].get(field, None)
+        if f is not None:
+            if f['datatype'] == 'composite' and f['#value#'] is None:
+                self.get(field)
+
     def deepcopy(self, class_generator=lambda : Metadata(None)):
         ''' Do not use this method unless you know what you are doing, if you
         want to create a simple clone of this object, use :meth:`deepcopy_metadata`
         instead. Class_generator must be a function that returns an instance
         of Metadata or a subclass of it.'''
+        # We don't need to evaluate all the composites here because we
+        # are returning a "real" Metadata instance that has __get_attribute__.
         m = class_generator()
         if not isinstance(m, Metadata):
             return None
@@ -207,6 +229,8 @@ class Metadata(object):
         return m
 
     def deepcopy_metadata(self):
+        # We don't need to evaluate all the composites here because we
+        # are returning a "real" Metadata instance that has __get_attribute__.
         m = Metadata(None)
         object.__setattr__(m, '_data', copy.deepcopy(object.__getattribute__(self, '_data')))
         return m
@@ -218,6 +242,8 @@ class Metadata(object):
             return default
 
     def get_extra(self, field, default=None):
+        # Don't need to evaluate all composites because a composite can't have
+        # an extra value
         _data = object.__getattribute__(self, '_data')
         if field in _data['user_metadata']:
             try:
@@ -237,8 +263,7 @@ class Metadata(object):
         needed is large. Also, we don't want any manipulations of the returned
         dict to show up in the book.
         '''
-        ans = object.__getattribute__(self,
-            '_data')['identifiers']
+        ans = object.__getattribute__(self, '_data')['identifiers']
         if not ans:
             ans = {}
         return copy.deepcopy(ans)
@@ -263,16 +288,14 @@ class Metadata(object):
         typ, val = self._clean_identifier(typ, val)
         if not typ:
             return
-        identifiers = object.__getattribute__(self,
-            '_data')['identifiers']
+        identifiers = object.__getattribute__(self, '_data')['identifiers']
 
         identifiers.pop(typ, None)
         if val:
             identifiers[typ] = val
 
     def has_identifier(self, typ):
-        identifiers = object.__getattribute__(self,
-            '_data')['identifiers']
+        identifiers = object.__getattribute__(self, '_data')['identifiers']
         return typ in identifiers
 
     # field-oriented interface. Intended to be the same as in LibraryDatabase
@@ -363,6 +386,9 @@ class Metadata(object):
         return a dict containing all the custom field metadata associated with
         the book.
         '''
+        # Must evaluate all composites because we are returning a dict, not a
+        # Metadata instance
+        self._evaluate_all_composites()
         _data = object.__getattribute__(self, '_data')
         user_metadata = _data['user_metadata']
         if not make_copy:
@@ -378,9 +404,12 @@ class Metadata(object):
         None. field is the key name, not the label. Return a copy if requested,
         just in case the user wants to change values in the dict.
         '''
-        _data = object.__getattribute__(self, '_data')
-        _data = _data['user_metadata']
+        _data = object.__getattribute__(self, '_data')['user_metadata']
         if field in _data:
+            # Must evaluate the field because it might be a composite. It won't
+            # be evaluated on demand because we are returning its dict, not a
+            # Metadata instance
+            self._evaluate_composite(field)
             if make_copy:
                 return copy.deepcopy(_data[field])
             return _data[field]
@@ -432,6 +461,19 @@ class Metadata(object):
             _data = object.__getattribute__(self, '_data')
             _data['user_metadata'][field] = m
 
+    def remove_stale_user_metadata(self, other_mi):
+        '''
+        Remove user metadata keys (custom column keys) if they
+        don't exist in 'other_mi', which must be a metadata object
+        '''
+        me = self.get_all_user_metadata(make_copy=False)
+        other = set(other_mi.custom_field_keys())
+        new = {}
+        for k,v in me.items():
+            if k in other:
+                new[k] = v
+        self.set_all_user_metadata(new)
+
     def template_to_attribute(self, other, ops):
         '''
         Takes a list [(src,dest), (src,dest)], evaluates the template in the
@@ -530,11 +572,11 @@ class Metadata(object):
 
             if getattr(other, 'cover_data', False):
                 other_cover = other.cover_data[-1]
-                self_cover = self.cover_data[-1] if self.cover_data else ''
+                self_cover = self.cover_data[-1] if self.cover_data else b''
                 if not self_cover:
-                    self_cover = ''
+                    self_cover = b''
                 if not other_cover:
-                    other_cover = ''
+                    other_cover = b''
                 if len(other_cover) > len(self_cover):
                     self.cover_data = other.cover_data
 
@@ -543,6 +585,8 @@ class Metadata(object):
                     meta = other.get_user_metadata(x, make_copy=True)
                     if meta is not None:
                         self_tags = self.get(x, [])
+                        if isinstance(self_tags, string_or_bytes):
+                            self_tags = []
                         self.set_user_metadata(x, meta)  # get... did the deepcopy
                         other_tags = other.get(x, [])
                         if meta['datatype'] == 'text' and meta['is_multiple']:
@@ -594,7 +638,7 @@ class Metadata(object):
         v = self.series_index if val is None else val
         try:
             x = float(v)
-        except (ValueError, TypeError):
+        except Exception:
             x = 1
         return fmt_sidx(x)
 
@@ -607,14 +651,14 @@ class Metadata(object):
         return authors_to_string(self.authors)
 
     def format_tags(self):
-        return u', '.join([unicode_type(t) for t in sorted(self.tags, key=sort_key)])
+        return ', '.join([str(t) for t in sorted(self.tags, key=sort_key)])
 
-    def format_rating(self, v=None, divide_by=1.0):
+    def format_rating(self, v=None, divide_by=1):
         if v is None:
             if self.rating is not None:
-                return unicode_type(self.rating/divide_by)
-            return u'None'
-        return unicode_type(v/divide_by)
+                return str(self.rating/divide_by)
+            return 'None'
+        return str(v/divide_by)
 
     def format_field(self, key, series_with_index=True):
         '''
@@ -638,15 +682,15 @@ class Metadata(object):
             if cmeta and cmeta['datatype'] == 'series':
                 if self.get(tkey):
                     res = self.get_extra(tkey)
-                    return (unicode_type(cmeta['name']+'_index'),
+                    return (str(cmeta['name']+'_index'),
                             self.format_series_index(res), res, cmeta)
                 else:
-                    return (unicode_type(cmeta['name']+'_index'), '', '', cmeta)
+                    return (str(cmeta['name']+'_index'), '', '', cmeta)
 
         if key in self.custom_field_keys():
             res = self.get(key, None)       # get evaluates all necessary composites
             cmeta = self.get_user_metadata(key, make_copy=False)
-            name = unicode_type(cmeta['name'])
+            name = str(cmeta['name'])
             if res is None or res == '':    # can't check "not res" because of numeric fields
                 return (name, res, None, None)
             orig_res = res
@@ -662,14 +706,14 @@ class Metadata(object):
             elif datatype == 'bool':
                 res = _('Yes') if res else _('No')
             elif datatype == 'rating':
-                res = u'%.2g'%(res/2.0)
+                res = '%.2g'%(res/2)
             elif datatype in ['int', 'float']:
                 try:
                     fmt = cmeta['display'].get('number_format', None)
                     res = fmt.format(res)
                 except:
                     pass
-            return (name, unicode_type(res), orig_res, cmeta)
+            return (name, str(res), orig_res, cmeta)
 
         # convert top-level ids into their value
         if key in TOP_LEVEL_IDENTIFIERS:
@@ -683,11 +727,11 @@ class Metadata(object):
         if fmkey in field_metadata and field_metadata[fmkey]['kind'] == 'field':
             res = self.get(key, None)
             fmeta = field_metadata[fmkey]
-            name = unicode_type(fmeta['name'])
+            name = str(fmeta['name'])
             if res is None or res == '':
                 return (name, res, None, None)
             orig_res = res
-            name = unicode_type(fmeta['name'])
+            name = str(fmeta['name'])
             datatype = fmeta['datatype']
             if key == 'authors':
                 res = authors_to_string(res)
@@ -702,10 +746,10 @@ class Metadata(object):
             elif datatype == 'datetime':
                 res = format_date(res, fmeta['display'].get('date_format','dd MMM yyyy'))
             elif datatype == 'rating':
-                res = u'%.2g'%(res/2.0)
+                res = '%.2g'%(res/2)
             elif key == 'size':
                 res = human_readable(res)
-            return (name, unicode_type(res), orig_res, fmeta)
+            return (name, str(res), orig_res, fmeta)
 
         return (None, None, None, None)
 
@@ -714,12 +758,12 @@ class Metadata(object):
         A string representation of this object, suitable for printing to
         console
         '''
-        from calibre.utils.date import isoformat
         from calibre.ebooks.metadata import authors_to_string
+        from calibre.utils.date import isoformat
         ans = []
 
         def fmt(x, y):
-            ans.append(u'%-20s: %s'%(unicode_type(x), unicode_type(y)))
+            ans.append('%-20s: %s'%(str(x), str(y)))
 
         fmt('Title', self.title)
         if self.title_sort:
@@ -733,22 +777,22 @@ class Metadata(object):
         if getattr(self, 'book_producer', False):
             fmt('Book Producer', self.book_producer)
         if self.tags:
-            fmt('Tags', u', '.join([unicode_type(t) for t in self.tags]))
+            fmt('Tags', ', '.join([str(t) for t in self.tags]))
         if self.series:
             fmt('Series', self.series + ' #%s'%self.format_series_index())
         if not self.is_null('languages'):
             fmt('Languages', ', '.join(self.languages))
         if self.rating is not None:
-            fmt('Rating', (u'%.2g'%(float(self.rating)/2.0)) if self.rating
-                    else u'')
+            fmt('Rating', ('%.2g'%(float(self.rating)/2)) if self.rating
+                    else '')
         if self.timestamp is not None:
             fmt('Timestamp', isoformat(self.timestamp))
         if self.pubdate is not None:
             fmt('Published', isoformat(self.pubdate))
         if self.rights is not None:
-            fmt('Rights', unicode_type(self.rights))
+            fmt('Rights', str(self.rights))
         if self.identifiers:
-            fmt('Identifiers', u', '.join(['%s:%s'%(k, v) for k, v in
+            fmt('Identifiers', ', '.join(['%s:%s'%(k, v) for k, v in
                 iteritems(self.identifiers)]))
         if self.comments:
             fmt('Comments', self.comments)
@@ -757,8 +801,8 @@ class Metadata(object):
             val = self.get(key, None)
             if val:
                 (name, val) = self.format_field(key)
-                fmt(name, unicode_type(val))
-        return u'\n'.join(ans)
+                fmt(name, str(val))
+        return '\n'.join(ans)
 
     def to_html(self):
         '''
@@ -766,38 +810,32 @@ class Metadata(object):
         '''
         from calibre.ebooks.metadata import authors_to_string
         from calibre.utils.date import isoformat
-        ans = [(_('Title'), unicode_type(self.title))]
+        ans = [(_('Title'), str(self.title))]
         ans += [(_('Author(s)'), (authors_to_string(self.authors) if self.authors else _('Unknown')))]
-        ans += [(_('Publisher'), unicode_type(self.publisher))]
-        ans += [(_('Producer'), unicode_type(self.book_producer))]
-        ans += [(_('Comments'), unicode_type(self.comments))]
-        ans += [('ISBN', unicode_type(self.isbn))]
-        ans += [(_('Tags'), u', '.join([unicode_type(t) for t in self.tags]))]
+        ans += [(_('Publisher'), str(self.publisher))]
+        ans += [(_('Producer'), str(self.book_producer))]
+        ans += [(_('Comments'), str(self.comments))]
+        ans += [('ISBN', str(self.isbn))]
+        ans += [(_('Tags'), ', '.join([str(t) for t in self.tags]))]
         if self.series:
-            ans += [(_('Series'), unicode_type(self.series) + ' #%s'%self.format_series_index())]
-        ans += [(_('Languages'), u', '.join(self.languages))]
+            ans += [(ngettext('Series', 'Series', 1), str(self.series) + ' #%s'%self.format_series_index())]
+        ans += [(_('Languages'), ', '.join(self.languages))]
         if self.timestamp is not None:
-            ans += [(_('Timestamp'), unicode_type(isoformat(self.timestamp, as_utc=False, sep=' ')))]
+            ans += [(_('Timestamp'), str(isoformat(self.timestamp, as_utc=False, sep=' ')))]
         if self.pubdate is not None:
-            ans += [(_('Published'), unicode_type(isoformat(self.pubdate, as_utc=False, sep=' ')))]
+            ans += [(_('Published'), str(isoformat(self.pubdate, as_utc=False, sep=' ')))]
         if self.rights is not None:
-            ans += [(_('Rights'), unicode_type(self.rights))]
+            ans += [(_('Rights'), str(self.rights))]
         for key in self.custom_field_keys():
             val = self.get(key, None)
             if val:
                 (name, val) = self.format_field(key)
                 ans += [(name, val)]
         for i, x in enumerate(ans):
-            ans[i] = u'<tr><td><b>%s</b></td><td>%s</td></tr>'%x
-        return u'<table>%s</table>'%u'\n'.join(ans)
+            ans[i] = '<tr><td><b>%s</b></td><td>%s</td></tr>'%x
+        return '<table>%s</table>'%'\n'.join(ans)
 
-    if ispy3:
-        __str__ = __unicode__representation__
-    else:
-        __unicode__ = __unicode__representation__
-
-        def __str__(self):
-            return self.__unicode__().encode('utf-8')
+    __str__ = __unicode__representation__
 
     def __nonzero__(self):
         return bool(self.title or self.author or self.comments or self.tags)
@@ -816,8 +854,12 @@ def field_from_string(field, raw, field_metadata):
     elif dt == 'rating':
         val = float(raw) * 2
     elif dt == 'datetime':
-        from calibre.utils.date import parse_only_date
-        val = parse_only_date(raw)
+        from calibre.utils.iso8601 import parse_iso8601
+        try:
+            val = parse_iso8601(raw, require_aware=True)
+        except Exception:
+            from calibre.utils.date import parse_only_date
+            val = parse_only_date(raw)
     elif dt == 'bool':
         if raw.lower() in {'true', 'yes', 'y'}:
             val = True
