@@ -523,10 +523,11 @@ def add_item_specific_entries(menu, data, book_info, copy_menu, search_menu):
                 v = data.get('original_value') or data.get('value')
                 copy_menu.addAction(QIcon.ic('edit-copy.png'), _('The text: {}').format(v),
                                         lambda: QApplication.instance().clipboard().setText(v))
-            ac = book_info.remove_item_action
-            ac.data = (field, remove_value, book_id)
-            ac.setText(_('Remove %s from this book') % escape_for_menu(remove_name or data.get('original_value') or value))
-            menu.addAction(ac)
+            if field != 'size':
+                ac = book_info.remove_item_action
+                ac.data = (field, remove_value, book_id)
+                ac.setText(_('Remove %s from this book') % escape_for_menu(remove_name or data.get('original_value') or value))
+                menu.addAction(ac)
             # See if we need to add a click associated link menu line
             link_map = get_gui().current_db.new_api.get_all_link_maps_for_book(data.get('book_id', -1))
             link = link_map.get(field, {}).get(value)
@@ -998,6 +999,7 @@ class BookInfo(HTMLDisplay):
         HTMLDisplay.__init__(self, parent=parent, save_resources_in_document=False)
         self.vertical = vertical
         self.last_rendered_html = '', '', ''
+        self.base_url_for_current_book = None
         self.anchor_clicked.connect(self.link_activated)
         for x, icon in [
             ('remove_format', 'trash.png'), ('save_format', 'save.png'),
@@ -1088,8 +1090,13 @@ class BookInfo(HTMLDisplay):
 
     def show_data(self, mi):
         html, table, comments = self.last_rendered_html = render_html(mi, self.vertical, self.parent())
+        path = getattr(mi, 'path', None)
+        self.base_url_for_current_book = QUrl.fromLocalFile(os.path.join(path, 'metadata.opf')) if path else None
         set_html(mi, html, self)
 
+    def get_base_qurl(self):
+        return self.base_url_for_current_book
+
     def process_external_css(self, css):
         return resolve_colors(css)
 
