@@ -1,26 +1,24 @@
-
-
 __license__   = 'GPL v3'
 __copyright__ = '2008, Kovid Goyal <kovid at kovidgoyal.net>'
 
-import re, copy
+import copy
+import re
 from datetime import date
-
-from PyQt5.Qt import (
-    QDialog, QDialogButtonBox, QFrame, QLabel, QComboBox, QIcon, QVBoxLayout, Qt,
-    QSize, QHBoxLayout, QTabWidget, QLineEdit, QWidget, QGroupBox, QFormLayout,
-    QSpinBox, QRadioButton
+from qt.core import (
+    QComboBox, QDialog, QDialogButtonBox, QFormLayout, QFrame, QGroupBox, QHBoxLayout,
+    QIcon, QLabel, QLineEdit, QPushButton, QRadioButton, QSize, QSpinBox, Qt,
+    QTabWidget, QToolButton, QVBoxLayout, QWidget,
 )
 
 from calibre import strftime
-from calibre.library.caches import CONTAINS_MATCH, EQUALS_MATCH
 from calibre.gui2 import gprefs
 from calibre.gui2.complete2 import EditWithComplete
-from calibre.utils.icu import sort_key
+from calibre.library.caches import CONTAINS_MATCH, EQUALS_MATCH, REGEXP_MATCH
+from calibre.startup import connect_lambda
 from calibre.utils.config import tweaks
 from calibre.utils.date import now
+from calibre.utils.icu import sort_key
 from calibre.utils.localization import localize_user_manual_link
-from polyglot.builtins import unicode_type, range, map
 
 box_values = {}
 last_matchkind = CONTAINS_MATCH
@@ -34,12 +32,14 @@ def init_dateop(cb):
             ('>', _('after')),
             ('<=', _('before or equal to')),
             ('>=', _('after or equal to')),
+            ('s', _('is set')),
+            ('u', _('is unset')),
     ]:
         cb.addItem(desc, op)
 
 
 def current_dateop(cb):
-    return unicode_type(cb.itemData(cb.currentIndex()) or '')
+    return str(cb.itemData(cb.currentIndex()) or '')
 
 
 def create_msg_label(self):
@@ -67,6 +67,7 @@ def create_match_kind(self):
         _("Contains: the word or phrase matches anywhere in the metadata field"),
         _("Equals: the word or phrase must match the entire metadata field"),
         _("Regular expression: the expression must match anywhere in the metadata field"),
+        _("Character variant: 'contains' with accents ignored and punctuation significant")
     ])
     l = QHBoxLayout()
     l.addWidget(la), l.addWidget(m)
@@ -92,19 +93,21 @@ def create_adv_tab(self):
     l.addWidget(w.g1), l.addWidget(w.g2), l.addStretch(10)
 
     w.g1.l = l = QFormLayout(w.g1)
-    l.setFieldGrowthPolicy(l.AllNonFixedFieldsGrow)
+    l.setFieldGrowthPolicy(QFormLayout.FieldGrowthPolicy.AllNonFixedFieldsGrow)
     for key, text in (
             ('all', _("A&ll these words:")),
             ('phrase', _("&This exact phrase:")),
             ('any', _("O&ne or more of these words:")),
     ):
         le = QLineEdit(w)
+        le.setClearButtonEnabled(True)
         setattr(self, key, le)
         l.addRow(text, le)
 
     w.g2.l = l = QFormLayout(w.g2)
-    l.setFieldGrowthPolicy(l.AllNonFixedFieldsGrow)
+    l.setFieldGrowthPolicy(QFormLayout.FieldGrowthPolicy.AllNonFixedFieldsGrow)
     self.none = le = QLineEdit(w)
+    le.setClearButtonEnabled(True)
     l.addRow(_("Any of these &unwanted words:"), le)
 
 
@@ -113,9 +116,10 @@ def create_simple_tab(self, db):
     self.tab_widget.addTab(w, _("Titl&e/author/series..."))
 
     w.l = l = QFormLayout(w)
-    l.setFieldGrowthPolicy(l.AllNonFixedFieldsGrow)
+    l.setFieldGrowthPolicy(QFormLayout.FieldGrowthPolicy.AllNonFixedFieldsGrow)
 
     self.title_box = le = QLineEdit(w)
+    le.setClearButtonEnabled(True)
     le.setObjectName('title_box')
     le.setPlaceholderText(_('The title to search for'))
     l.addRow(_('&Title:'), le)
@@ -150,6 +154,7 @@ def create_simple_tab(self, db):
     self.general_combo.addItems(searchables)
     self.box_last_values = copy.deepcopy(box_values)
     self.general_box = le = QLineEdit(self)
+    le.setClearButtonEnabled(True)
     le.setObjectName('general_box')
     l.addRow(self.general_combo, le)
     if self.box_last_values:
@@ -161,8 +166,21 @@ def create_simple_tab(self, db):
                 self.general_combo.findText(self.box_last_values['general_index']))
 
 
+def toggle_date_conditions_visibility(self):
+    dcl = self.date_tab.date_condition_layouts
+    op = current_dateop(self.dateop_date)
+    visible = op not in 'su'
+    for l in dcl:
+        for i in range(l.count()):
+            x = l.itemAt(i)
+            w = x.widget()
+            if w is not None:
+                w.setVisible(visible)
+
+
 def create_date_tab(self, db):
     self.date_tab = w = QWidget(self.tab_widget)
+    w.date_condition_layouts = dcl = []
     self.tab_widget.addTab(w, _("&Date search"))
     w.l = l = QVBoxLayout(w)
 
@@ -189,11 +207,13 @@ def create_date_tab(self, db):
     h.addWidget(df)
     self.dateop_date = dd = add(_("date column for books whose &date is "), QComboBox(w))
     init_dateop(dd)
+    connect_lambda(dd.currentIndexChanged, self, toggle_date_conditions_visibility)
     w.la3 = la = QLabel('...')
     h.addWidget(la)
     h.addStretch(10)
 
     w.h2 = h = QHBoxLayout()
+    dcl.append(h)
     l.addLayout(h)
     self.sel_date = a(QRadioButton(_('&year'), w))
     self.date_year = dy = a(QSpinBox(w))
@@ -208,6 +228,7 @@ def create_date_tab(self, db):
     h.addStretch(10)
 
     w.h3 = h = QHBoxLayout()
+    dcl.append(h)
     l.addLayout(h)
     self.sel_daysago = a(QRadioButton('', w))
     self.date_daysago = da = a(QSpinBox(w))
@@ -219,6 +240,7 @@ def create_date_tab(self, db):
 
     w.h4 = h = QHBoxLayout()
     l.addLayout(h)
+    dcl.append(h)
     self.sel_human = a(QRadioButton('', w))
     self.date_human = dh = a(QComboBox(w))
     for val, text in [('today', _('Today')), ('yesterday', _('Yesterday')), ('thismonth', _('This month'))]:
@@ -232,6 +254,7 @@ def create_date_tab(self, db):
     h.addStretch(10)
 
     l.addStretch(10)
+    toggle_date_conditions_visibility(self)
 
 
 def create_template_tab(self):
@@ -239,19 +262,21 @@ def create_template_tab(self):
     self.tab_widget.addTab(w, _("&Template search"))
 
     w.l = l = QFormLayout(w)
-    l.setFieldGrowthPolicy(l.AllNonFixedFieldsGrow)
+    l.setFieldGrowthPolicy(QFormLayout.FieldGrowthPolicy.AllNonFixedFieldsGrow)
 
     self.template_value_box = le = QLineEdit(w)
+    le.setClearButtonEnabled(True)
     le.setObjectName('template_value_box')
     le.setPlaceholderText(_('The value to search for'))
     le.setToolTip('<p>' +
-                  _("You can use the search test specifications described "
+                  _("You can use the search specifications described "
                     "in the calibre documentation. For example, with Number "
-                    "comparisons you can the relational operators like '>=' etc. "
-                    "With Text comparisons you can use exact, contains "
-                    "or regular expression matches. With Date you can use "
-                    "today, yesterday, etc. Set/not set takes 'true' for set "
-                    "and 'false' for not set.") + '</p>')
+                    "comparisons you can use the relational operators like '>=' etc. "
+                    "With Text comparisons you can use contains (T), exact (=T), "
+                    "or regular expression matches (~T), where T is your text. "
+                    "With Date you can use 'today', 'yesterday', etc. When checking for "
+                    "Set use 'true' or 'yes'. When checking for Not set use 'false' "
+                    "or 'no'") + '</p>')
     l.addRow(_('Template &value:'), le)
 
     self.template_test_type_box = le = QComboBox(w)
@@ -269,13 +294,34 @@ def create_template_tab(self):
     self.template_program_box = le = TemplateLineEditor(self.tab_widget)
     le.setObjectName('template_program_box')
     le.setPlaceholderText(_('The template that generates the value'))
-    le.setToolTip(_('Right click to open a template editor'))
-    l.addRow(_('Tem&plate:'), le)
+    le.setToolTip('<p>' +
+                  _('Right click to open a template editor. <br>'
+                    'Technical note: the set of book ids already matched by '
+                    'previous search terms in the search expression is passed '
+                    'to the template in the global variables dictionary with the '
+                    'key "{0}". You can use the set to limit any work the '
+                    'template does to the set of books already matched, possibly '
+                    'improving performance.'
+                    ).format('_candidates')  + '</p>')
+    lo = QHBoxLayout()
+    lo.addWidget(le)
+    self.edit_template_button = tb = QToolButton()
+    tb.setIcon(QIcon.ic("edit_input.png"))
+    tb.setToolTip(_('Open template editor'))
+    lo.addWidget(tb)
+    self.template_layout_label = tll = QLabel(_('&Template:'))
+    tll.setBuddy(le)
+    l.addRow(tll, lo)
+
+    self.copy_current_template_search_button = le = QPushButton(_('&Copy the current search into the boxes'))
+    le.setObjectName('copy_current_template_search_button')
+    le.setToolTip(_('Use this button to retrieve and edit the current search'))
+    l.addRow('', le)
 
 
 def setup_ui(self, db):
     self.setWindowTitle(_("Advanced search"))
-    self.setWindowIcon(QIcon(I('search.png')))
+    self.setWindowIcon(QIcon.ic('search.png'))
     self.l = l = QVBoxLayout(self)
     self.h = h = QHBoxLayout()
     self.v = v = QVBoxLayout()
@@ -301,6 +347,14 @@ class SearchDialog(QDialog):
         QDialog.__init__(self, parent)
         setup_ui(self, db)
 
+        # Get metadata of some of the selected books to give to the template
+        # dialog to help test the template
+        from calibre.gui2.ui import get_gui
+        view = get_gui().library_view
+        rows = view.selectionModel().selectedRows()[0:10]  # Maximum of 10 books
+        mi = [db.new_api.get_proxy_metadata(db.data.index_to_id(x.row())) for x in rows]
+        self.template_program_box.set_mi(mi)
+
         current_tab = gprefs.get('advanced search dialog current tab', 0)
         self.tab_widget.setCurrentIndex(current_tab)
         if current_tab == 1:
@@ -316,8 +370,34 @@ class SearchDialog(QDialog):
                       gprefs.get('advanced_search_template_tab_value_field', ''))
             self.template_test_type_box.setCurrentIndex(
                       int(gprefs.get('advanced_search_template_tab_test_field', '0')))
+        self.current_search_text = get_gui().search.current_text
+        if self.current_search_text.startswith('template:'):
+            self.current_search_text = self.current_search_text[len('template:'):]
+            if self.current_search_text.startswith('"""'):
+                self.current_search_text = self.current_search_text[3:-3]
+            elif self.current_search_text.startswith('"'):
+                # This is a hack to try to compensate for template searches
+                # that were surrounded with quotes not docstrings. If there is
+                # escaping in the quoted string it won't be right because the
+                # final output will be docstring encoded.
+                self.current_search_text = self.current_search_text[1:-1]
+            self.copy_current_template_search_button.setEnabled(True)
+        else:
+            self.copy_current_template_search_button.setEnabled(False)
+        self.copy_current_template_search_button.clicked.connect(self.retrieve_template_search)
+        self.edit_template_button.clicked.connect(lambda:self.template_program_box.open_editor())
         self.resize(self.sizeHint())
 
+    def retrieve_template_search(self):
+        template, sep, query = re.split('#@#:([tdnb]):', self.current_search_text, flags=re.IGNORECASE)
+        self.template_value_box.setText(query)
+        cb = self.template_test_type_box
+        for idx in range(0, cb.count()):
+            if sep == str(cb.itemData(idx)):
+                cb.setCurrentIndex(idx)
+                break
+        self.template_program_box.setText(template)
+
     def save_state(self):
         gprefs['advanced search dialog current tab'] = \
             self.tab_widget.currentIndex()
@@ -327,11 +407,11 @@ class SearchDialog(QDialog):
                 gprefs.set('advanced_search_simple_tab_focused_field', fw.objectName())
         elif self.tab_widget.currentIndex() == 3:
             gprefs.set('advanced_search_template_tab_program_field',
-                       unicode_type(self.template_program_box.text()))
+                       str(self.template_program_box.text()))
             gprefs.set('advanced_search_template_tab_value_field',
-                       unicode_type(self.template_value_box.text()))
+                       str(self.template_value_box.text()))
             gprefs.set('advanced_search_template_tab_test_field',
-                       unicode_type(self.template_test_type_box.currentIndex()))
+                       str(self.template_test_type_box.currentIndex()))
 
     def accept(self):
         self.save_state()
@@ -369,21 +449,22 @@ class SearchDialog(QDialog):
                 self.date_search_string, self.template_search_string)[i]()
 
     def template_search_string(self):
-        template = unicode_type(self.template_program_box.text())
-        value = unicode_type(self.template_value_box.text()).replace('"', '\\"')
-        if template and value:
-            cb = self.template_test_type_box
-            op =  unicode_type(cb.itemData(cb.currentIndex()))
-            l = '{0}#@#:{1}:{2}'.format(template, op, value)
-            return 'template:"' + l + '"'
-        return ''
+        template = str(self.template_program_box.text())
+        value = str(self.template_value_box.text())
+        cb = self.template_test_type_box
+        op =  str(cb.itemData(cb.currentIndex()))
+        l = f'{template}#@#:{op}:{value}'
+        # Use docstring quoting (super-quoting) to avoid problems with escaping
+        return 'template:"""' + l + '"""'
 
     def date_search_string(self):
-        field = unicode_type(self.date_field.itemData(self.date_field.currentIndex()) or '')
+        field = str(self.date_field.itemData(self.date_field.currentIndex()) or '')
         op = current_dateop(self.dateop_date)
-        prefix = '%s:%s' % (field, op)
+        if op in 'su':
+            return f'{field}:{"true" if op == "s" else "false"}'
+        prefix = f'{field}:{op}'
         if self.sel_date.isChecked():
-            ans = '%s%s' % (prefix, self.date_year.value())
+            ans = f'{prefix}{self.date_year.value()}'
             m = self.date_month.itemData(self.date_month.currentIndex())
             if m > 0:
                 ans += '-%s' % m
@@ -394,8 +475,8 @@ class SearchDialog(QDialog):
         if self.sel_daysago.isChecked():
             val = self.date_daysago.value()
             val *= {0:1, 1:7, 2:30, 3:365}[self.date_ago_type.currentIndex()]
-            return '%s%sdaysago' % (prefix, val)
-        return '%s%s' % (prefix, unicode_type(self.date_human.itemData(self.date_human.currentIndex()) or ''))
+            return f'{prefix}{val}daysago'
+        return '{}{}'.format(prefix, str(self.date_human.itemData(self.date_human.currentIndex()) or ''))
 
     def adv_search_string(self):
         mk = self.matchkind.currentIndex()
@@ -403,9 +484,11 @@ class SearchDialog(QDialog):
             self.mc = ''
         elif mk == EQUALS_MATCH:
             self.mc = '='
-        else:
+        elif mk == REGEXP_MATCH:
             self.mc = '~'
-        all, any, phrase, none = map(lambda x: unicode_type(x.text()),
+        else:
+            self.mc = '^'
+        all, any, phrase, none = map(lambda x: str(x.text()),
                 (self.all, self.any, self.phrase, self.none))
         all, any, none = map(self.tokens, (all, any, none))
         phrase = phrase.strip()
@@ -427,11 +510,11 @@ class SearchDialog(QDialog):
         return ans
 
     def token(self):
-        txt = unicode_type(self.text.text()).strip()
+        txt = str(self.text.text()).strip()
         if txt:
             if self.negate.isChecked():
                 txt = '!'+txt
-            tok = self.FIELDS[unicode_type(self.field.currentText())]+txt
+            tok = self.FIELDS[str(self.field.currentText())]+txt
             if re.search(r'\s', tok):
                 tok = '"%s"'%tok
             return tok
@@ -442,40 +525,42 @@ class SearchDialog(QDialog):
             self.mc = ''
         elif mk == EQUALS_MATCH:
             self.mc = '='
-        else:
+        elif mk == REGEXP_MATCH:
             self.mc = '~'
+        else:
+            self.mc = '^'
 
         ans = []
         self.box_last_values = {}
-        title = unicode_type(self.title_box.text()).strip()
+        title = str(self.title_box.text()).strip()
         self.box_last_values['title_box'] = title
         if title:
             ans.append('title:"' + self.mc + title + '"')
-        author = unicode_type(self.authors_box.text()).strip()
+        author = str(self.authors_box.text()).strip()
         self.box_last_values['authors_box'] = author
         if author:
             ans.append('author:"' + self.mc + author + '"')
-        series = unicode_type(self.series_box.text()).strip()
+        series = str(self.series_box.text()).strip()
         self.box_last_values['series_box'] = series
         if series:
             ans.append('series:"' + self.mc + series + '"')
 
-        tags = unicode_type(self.tags_box.text())
+        tags = str(self.tags_box.text())
         self.box_last_values['tags_box'] = tags
         tags = [t.strip() for t in tags.split(',') if t.strip()]
         if tags:
             tags = ['tags:"' + self.mc + t + '"' for t in tags]
             ans.append('(' + ' or '.join(tags) + ')')
-        general = unicode_type(self.general_box.text())
+        general = str(self.general_box.text())
         self.box_last_values['general_box'] = general
-        general_index = unicode_type(self.general_combo.currentText())
+        general_index = str(self.general_combo.currentText())
         self.box_last_values['general_index'] = general_index
         global box_values
         global last_matchkind
         box_values = copy.deepcopy(self.box_last_values)
         last_matchkind = mk
         if general:
-            ans.append(unicode_type(self.general_combo.currentText()) + ':"' +
+            ans.append(str(self.general_combo.currentText()) + ':"' +
                     self.mc + general + '"')
         if ans:
             return ' and '.join(ans)
@@ -488,6 +573,6 @@ if __name__ == '__main__':
     from calibre.gui2 import Application
     app = Application([])
     d = SearchDialog(None, db)
-    d.exec_()
+    d.exec()
 
     print(d.search_string())
