@@ -10,9 +10,10 @@ from contextlib import suppress
 from functools import partial
 from itertools import count
 from qt.core import (
-    QAbstractItemModel, QAbstractItemView, QCheckBox, QDialog, QDialogButtonBox, QFont,
-    QHBoxLayout, QIcon, QLabel, QMenu, QModelIndex, QPixmap, QPushButton, QRect, QSize,
-    QSplitter, QStackedWidget, Qt, QTreeView, QVBoxLayout, QWidget, pyqtSignal,
+    QAbstractItemModel, QAbstractItemView, QAction, QCheckBox, QDialog,
+    QDialogButtonBox, QFont, QHBoxLayout, QIcon, QKeySequence, QLabel, QMenu,
+    QModelIndex, QPixmap, QPushButton, QRect, QSize, QSplitter, QStackedWidget, Qt,
+    QTreeView, QVBoxLayout, QWidget, pyqtSignal,
 )
 from threading import Event, Thread
 
@@ -46,7 +47,9 @@ def jump_to_book(book_id, parent=None):
     gui = get_gui()
     if gui is not None:
         parent = parent or gui
-        if not gui.library_view.select_rows((book_id,)):
+        if gui.library_view.select_rows((book_id,)):
+            gui.raise_and_focus()
+        else:
             if gprefs['fts_library_restrict_books']:
                 error_dialog(parent, _('Not found'), _('This book was not found in the calibre library'), show=True)
             else:
@@ -456,9 +459,9 @@ class ResultsView(QTreeView):
             m.addAction(QIcon.ic('marked.png'), _('Mark this book in the library'), partial(mark_books, results.book_id))
             if match is not None:
                 match = index.row()
-                m.addAction(QIcon.ic('view.png'), _('Open this book at this search result'), partial(open_book, results, match_index=match))
+                m.addAction(QIcon.ic('view.png'), _('View this book at this search result'), partial(open_book, results, match_index=match))
             else:
-                m.addAction(QIcon.ic('view.png'), _('Open this book'), partial(open_book, results))
+                m.addAction(QIcon.ic('view.png'), _('View this book'), partial(open_book, results))
         m.addSeparator()
         m.addAction(QIcon.ic('plus.png'), _('Expand all'), self.expandAll)
         m.addAction(QIcon.ic('minus.png'), _('Collapse all'), self.collapseAll)
@@ -587,6 +590,9 @@ class ResultDetails(QWidget):
 
     def __init__(self, parent=None):
         super().__init__(parent)
+        self.jump_action = ac = QAction(self)
+        ac.triggered.connect(self.jump_to_current_book)
+        ac.setShortcut(QKeySequence('Ctrl+S', QKeySequence.SequenceFormat.PortableText))
         self.key = None
         self.pixmap_label = pl = QLabel(self)
         pl.setScaledContents(True)
@@ -624,6 +630,10 @@ class ResultDetails(QWidget):
                     ' to see updated results.'), show=True)
                 self.remove_book_from_results.emit(self.current_book_id)
 
+    def jump_to_current_book(self):
+        if self.current_book_id > -1:
+            jump_to_book(self.current_book_id)
+
     def results_anchor_clicked(self, url):
         if self.current_book_id > 0 and url.scheme() == 'book':
             book_id, result_num, fmt = url.path().strip('/').split('/')
@@ -684,7 +694,8 @@ class ResultDetails(QWidget):
             text += '<p>' + _('{series_index} of {series}').format(series_index=sidx, series=series) + '</p>'
         ict = '<img valign="bottom" src="calibre-icon:///{}" width=16 height=16>'
         text += '<p><a href="calibre://jump" title="{1}">{2}\xa0{0}</a>\xa0\xa0\xa0 '.format(
-            _('Select'), '<p>' + _('Scroll to this book in the calibre library book list and select it.'), ict.format('lt.png'))
+            _('Select'), '<p>' + _('Scroll to this book in the calibre library book list and select it [{}]').format(
+                self.jump_action.shortcut().toString(QKeySequence.SequenceFormat.NativeText)), ict.format('lt.png'))
         text += '<a href="calibre://mark" title="{1}">{2}\xa0{0}</a></p>'.format(
             _('Mark'), '<p>' + _(
                 'Put a pin on this book in the calibre library, for future reference.'
@@ -854,6 +865,10 @@ class ResultsPanel(QWidget):
         if st is not None:
             s.restoreState(st)
 
+    @property
+    def jump_to_current_book_action(self):
+        return self.details.result_details.jump_action
+
     def view_current_result(self):
         return self.results_view.view_current_result()
 
