@@ -13,12 +13,14 @@ from qt.core import (
     QAbstractItemView, QDialog, QDialogButtonBox, QDrag, QEvent, QFont, QFontMetrics,
     QGridLayout, QHeaderView, QIcon, QItemSelection, QItemSelectionModel, QLabel, QMenu,
     QMimeData, QModelIndex, QPoint, QPushButton, QSize, QSpinBox, QStyle,
-    QStyleOptionHeader, Qt, QTableView, QUrl, pyqtSignal,
+    QStyleOptionHeader, Qt, QTableView, QTimer, QUrl, pyqtSignal,
 )
 
 from calibre import force_unicode
 from calibre.constants import filesystem_encoding, islinux
-from calibre.gui2 import FunctionDispatcher, error_dialog, gprefs
+from calibre.gui2 import (
+    BOOK_DETAILS_DISPLAY_DEBOUNCE_DELAY, FunctionDispatcher, error_dialog, gprefs,
+)
 from calibre.gui2.dialogs.enum_values_edit import EnumValuesEdit
 from calibre.gui2.gestures import GestureManager
 from calibre.gui2.library import DEFAULT_SORT
@@ -27,9 +29,9 @@ from calibre.gui2.library.alternate_views import (
 )
 from calibre.gui2.library.delegates import (
     CcBoolDelegate, CcCommentsDelegate, CcDateDelegate, CcEnumDelegate,
-    CcLongTextDelegate, CcNumberDelegate, CcSeriesDelegate, CcTemplateDelegate,
-    CcTextDelegate, CompleteDelegate, DateDelegate, LanguagesDelegate, PubDateDelegate,
-    RatingDelegate, SeriesDelegate, TextDelegate,
+    CcLongTextDelegate, CcMarkdownDelegate, CcNumberDelegate, CcSeriesDelegate,
+    CcTemplateDelegate, CcTextDelegate, CompleteDelegate, DateDelegate,
+    LanguagesDelegate, PubDateDelegate, RatingDelegate, SeriesDelegate, TextDelegate,
 )
 from calibre.gui2.library.models import BooksModel, DeviceBooksModel
 from calibre.gui2.pin_columns import PinTableView
@@ -361,9 +363,10 @@ class BooksView(QTableView):  # {{{
             elif tval == 'open_viewer':
                 wv.setEditTriggers(QAbstractItemView.EditTrigger.SelectedClicked|wv.editTriggers())
                 wv.doubleClicked.connect(parent.iactions['View'].view_triggered)
-            elif tval == 'show_book_details':
+            elif tval in ('show_book_details', 'show_locked_book_details'):
                 wv.setEditTriggers(QAbstractItemView.EditTrigger.SelectedClicked|wv.editTriggers())
-                wv.doubleClicked.connect(parent.iactions['Show Book Details'].show_book_info)
+                wv.doubleClicked.connect(partial(parent.iactions['Show Book Details'].show_book_info,
+                                                 locked=tval == 'show_locked_book_details'))
             elif tval == 'edit_metadata':
                 # Must not enable single-click to edit, or the field will remain
                 # open in edit mode underneath the edit metadata dialog
@@ -396,6 +399,7 @@ class BooksView(QTableView):  # {{{
         self.cc_text_delegate = CcTextDelegate(self)
         self.cc_series_delegate = CcSeriesDelegate(self)
         self.cc_longtext_delegate = CcLongTextDelegate(self)
+        self.cc_markdown_delegate = CcMarkdownDelegate(self)
         self.cc_enum_delegate = CcEnumDelegate(self)
         self.cc_bool_delegate = CcBoolDelegate(self)
         self.cc_comments_delegate = CcCommentsDelegate(self)
@@ -499,12 +503,6 @@ class BooksView(QTableView):  # {{{
             gprefs['book_list_split'] = self.pin_view.isVisible()
             self.save_state()
             return
-        if action in ('lock', 'unlock'):
-            val = action == 'unlock'
-            self.column_header.setSectionsMovable(val)
-            self.pin_view.column_header.setSectionsMovable(val)
-            gprefs.set('allow_column_movement_with_mouse', val)
-
         if not action or not column or not view:
             return
         try:
@@ -513,7 +511,12 @@ class BooksView(QTableView):  # {{{
             return
         h = view.column_header
 
-        if action == 'hide':
+        if action == 'lock':
+            val = not view.column_header.sectionsMovable()
+            self.column_header.setSectionsMovable(val)
+            self.pin_view.column_header.setSectionsMovable(val)
+            gprefs.set('allow_column_movement_with_mouse', val)
+        elif action == 'hide':
             if h.hiddenSectionCount() >= h.count():
                 return error_dialog(self, _('Cannot hide all columns'), _(
                     'You must not hide all columns'), show=True)
@@ -679,23 +682,23 @@ class BooksView(QTableView):  # {{{
             view.column_header_context_menu = self.create_context_menu(col, name, view)
         has_context_menu = hasattr(view, 'column_header_context_menu')
         if self.is_library_view and has_context_menu:
-            view.column_header_context_menu.addSeparator()
-            if not hasattr(view.column_header_context_menu, 'bl_split_action'):
-                view.column_header_context_menu.bl_split_action = view.column_header_context_menu.addAction(
-                        QIcon.ic('split.png'), 'xxx', partial(self.column_header_context_handler, action='split', column='title'))
-            ac = view.column_header_context_menu.bl_split_action
+            m = view.column_header_context_menu
+            m.addSeparator()
+            if not hasattr(m, 'bl_split_action'):
+                m.bl_split_action = m.addAction(QIcon.ic('split.png'), 'xxx',
+                            partial(self.column_header_context_handler, action='split', column='title'))
+            ac = m.bl_split_action
             if self.pin_view.isVisible():
                 ac.setText(_('Un-split the book list'))
             else:
                 ac.setText(_('Split the book list'))
-            if view.column_header.sectionsMovable():
-                view.column_header_context_menu.addAction(
-                        QIcon.ic('drm-locked.png'), _("Don't allow moving columns with the mouse"),
-                        partial(self.column_header_context_handler, action='lock'))
-            else:
-                view.column_header_context_menu.addAction(
-                        QIcon.ic('drm-unlocked.png'), _("Allow moving columns with the mouse"),
-                        partial(self.column_header_context_handler, action='unlock'))
+
+            ac = getattr(m, 'column_mouse_move_action', None)
+            if ac is None:
+                ac = m.column_mouse_move_action = m.addAction(_("Allow moving columns with the mouse"),
+                          partial(self.column_header_context_handler, action='lock', column=col, view=view))
+                ac.setCheckable(True)
+            ac.setChecked(view.column_header.sectionsMovable())
         if has_context_menu:
             view.column_header_context_menu.popup(view.column_header.mapToGlobal(pos))
     # }}}
@@ -745,6 +748,10 @@ class BooksView(QTableView):  # {{{
         gprefs[pname] = previous
         self.sort_by_named_field(field, previous[field])
 
+    def keyboardSearch(self, search):
+        if gprefs.get('allow_keyboard_search_in_library_views', True):
+            super().keyboardSearch(search)
+
     def sort_by_named_field(self, field, order, reset=True):
         if isinstance(order, Qt.SortOrder):
             order = order == Qt.SortOrder.AscendingOrder
@@ -1127,8 +1134,10 @@ class BooksView(QTableView):  # {{{
                     ctype = cc['display'].get('interpret_as', 'html')
                     if ctype == 'short-text':
                         set_item_delegate(colhead, self.cc_text_delegate)
-                    elif ctype in ('long-text', 'markdown'):
+                    elif ctype == 'long-text':
                         set_item_delegate(colhead, self.cc_longtext_delegate)
+                    elif ctype == 'markdown':
+                        set_item_delegate(colhead, self.cc_markdown_delegate)
                     else:
                         set_item_delegate(colhead, self.cc_comments_delegate)
                 elif cc['datatype'] == 'text':
@@ -1153,10 +1162,9 @@ class BooksView(QTableView):  # {{{
                 elif cc['datatype'] == 'enumeration':
                     set_item_delegate(colhead, self.cc_enum_delegate)
             else:
-                if colhead in self._model.editable_cols:
-                    dattr = colhead+'_delegate'
-                    delegate = colhead if hasattr(self, dattr) else 'text'
-                    set_item_delegate(colhead, getattr(self, delegate+'_delegate'))
+                dattr = colhead+'_delegate'
+                delegate = colhead if hasattr(self, dattr) else 'text'
+                set_item_delegate(colhead, getattr(self, delegate+'_delegate'))
 
         self.restore_state()
         self.set_ondevice_column_visibility()
@@ -1303,19 +1311,35 @@ class BooksView(QTableView):  # {{{
             vertical_offset = vh.offset()
             vertical_position = vh.sectionPosition(row)
             cell_height = vh.sectionSize(row)
-
             pos = 'center'
             if vertical_position - vertical_offset < 0 or cell_height > viewport_height:
                 pos = 'top'
             elif vertical_position - vertical_offset + cell_height > viewport_height:
                 pos = 'bottom'
             vsb = self.verticalScrollBar()
-            if pos == 'top':
-                vsb.setValue(vertical_position)
-            elif pos == 'bottom':
-                vsb.setValue(vertical_position - viewport_height + cell_height)
+
+            if self.verticalScrollMode() == QAbstractItemView.ScrollMode.ScrollPerPixel:
+                if pos == 'top':
+                    vsb.setValue(vertical_position)
+                elif pos == 'bottom':
+                    vsb.setValue(vertical_position - viewport_height + cell_height)
+                else:
+                    vsb.setValue(vertical_position - ((viewport_height - cell_height) // 2))
             else:
-                vsb.setValue(vertical_position - ((viewport_height - cell_height) // 2))
+                vertical_index = vh.visualIndex(row)
+                if pos in ('bottom', 'center'):
+                    h = (viewport_height // 2) if pos == 'center' else viewport_height
+                    y = cell_height
+                    while vertical_index > 0:
+                        y += vh.sectionSize(vh.logicalIndex(vertical_index -1))
+                        if y > h:
+                            break
+                        vertical_index -= 1
+                if vh.sectionsHidden():
+                    for s in range(vertical_index - 1, -1, -1):
+                        if vh.isSectionHidden(vh.logicalIndex(s)):
+                            vertical_index -= 1
+                vsb.setValue(vertical_index)
 
     @property
     def current_book(self):
@@ -1593,7 +1617,20 @@ class BooksView(QTableView):  # {{{
             self._model.search_done.connect(self.alternate_views.restore_current_book_state)
 
     def connect_to_book_display(self, bd):
-        self._model.new_bookdisplay_data.connect(bd)
+        self.book_display_callback = bd
+        self.connect_to_book_display_timer = t = QTimer(self)
+        t.setSingleShot(True)
+        t.timeout.connect(self._debounce_book_display)
+        t.setInterval(BOOK_DETAILS_DISPLAY_DEBOUNCE_DELAY)
+        self._model.new_bookdisplay_data.connect(self._timed_connect_to_book_display)
+
+    def _timed_connect_to_book_display(self, data):
+        self._book_display_data = data
+        self.connect_to_book_display_timer.start()
+
+    def _debounce_book_display(self):
+        data, self._book_display_data = self._book_display_data, None
+        self.book_display_callback(data)
 
     def search_done(self, ok):
         self._search_done(self, ok)
