@@ -1,5 +1,4 @@
 #!/usr/bin/env python
-# vim:fileencoding=UTF-8:ts=4:sw=4:sta:et:sts=4:ai
 # License: GPLv3 Copyright: 2010, Kovid Goyal <kovid at kovidgoyal.net>
 
 import errno
@@ -9,41 +8,37 @@ import os
 import sys
 import textwrap
 import time
-
-from PyQt5.Qt import (
-    QCheckBox, QComboBox, QDialog, QDialogButtonBox, QDoubleSpinBox, QFormLayout,
-    QFrame, QHBoxLayout, QIcon, QLabel, QLineEdit, QListWidget, QPlainTextEdit,
-    QPushButton, QScrollArea, QSize, QSizePolicy, QSpinBox, Qt, QTabWidget, QTimer,
-    QToolButton, QUrl, QVBoxLayout, QWidget, pyqtSignal
+from qt.core import (
+    QApplication, QCheckBox, QComboBox, QDialog, QDialogButtonBox, QDoubleSpinBox,
+    QFormLayout, QFrame, QHBoxLayout, QIcon, QLabel, QLayout, QLineEdit, QListWidget,
+    QPlainTextEdit, QPushButton, QScrollArea, QSize, QSizePolicy, QSpinBox, Qt,
+    QTabWidget, QTimer, QToolButton, QUrl, QVBoxLayout, QWidget, pyqtSignal, sip,
 )
 
 from calibre import as_unicode
-from calibre.constants import isportable, iswindows, plugins
+from calibre.constants import isportable, iswindows
 from calibre.gui2 import (
-    choose_files, choose_save_file, config, error_dialog, gprefs, info_dialog,
-    open_url, warning_dialog
+    choose_files, choose_save_file, config, error_dialog, gprefs, info_dialog, open_url,
+    warning_dialog,
 )
 from calibre.gui2.preferences import AbortCommit, ConfigWidgetBase, test_widget
 from calibre.gui2.widgets import HistoryLineEdit
 from calibre.srv.code import custom_list_template as default_custom_list_template
 from calibre.srv.embedded import custom_list_template, search_the_net_urls
-from calibre.srv.loop import parse_trusted_ips
 from calibre.srv.library_broker import load_gui_libraries
+from calibre.srv.loop import parse_trusted_ips
 from calibre.srv.opts import change_settings, options, server_config
 from calibre.srv.users import (
-    UserManager, create_user_data, validate_password, validate_username
+    UserManager, create_user_data, validate_password, validate_username,
 )
 from calibre.utils.icu import primary_sort_key
+from calibre.utils.localization import ngettext
 from calibre.utils.shared_file import share_open
-from polyglot.builtins import as_bytes, unicode_type
-
-try:
-    from PyQt5 import sip
-except ImportError:
-    import sip
-
+from polyglot.builtins import as_bytes
 
 if iswindows and not isportable:
+    from calibre_extensions import winutil
+
     def get_exe():
         exe_base = os.path.abspath(os.path.dirname(sys.executable))
         exe = os.path.join(exe_base, 'calibre.exe')
@@ -52,7 +47,6 @@ if iswindows and not isportable:
         return exe
 
     def startup_shortcut_path():
-        winutil = plugins['winutil'][0]
         startup_path = winutil.special_folder_path(winutil.CSIDL_STARTUP)
         return os.path.join(startup_path, "calibre.lnk")
 
@@ -61,14 +55,14 @@ if iswindows and not isportable:
         if args:
             quoted_args = []
             for arg in args:
-                quoted_args.append('"{}"'.format(arg))
+                quoted_args.append(f'"{arg}"')
             quoted_args = ' '.join(quoted_args)
-        plugins['winutil'][0].manage_shortcut(shortcut_path, target, description, quoted_args)
+        winutil.manage_shortcut(shortcut_path, target, description, quoted_args)
 
     def shortcut_exists_at(shortcut_path, target):
         if not os.access(shortcut_path, os.R_OK):
             return False
-        name = plugins['winutil'][0].manage_shortcut(shortcut_path, None, None, None)
+        name = winutil.manage_shortcut(shortcut_path, None, None, None)
         if name is None:
             return False
         return os.path.normcase(os.path.abspath(name)) == os.path.normcase(os.path.abspath(target))
@@ -127,7 +121,7 @@ class Int(QSpinBox):
 
     def __init__(self, name, layout):
         QSpinBox.__init__(self)
-        self.setRange(0, 20000)
+        self.setRange(0, 99999)
         opt = options[name]
         self.valueChanged.connect(self.changed_signal.emit)
         init_opt(self, opt, layout)
@@ -173,7 +167,7 @@ class Text(QLineEdit):
         return self.text().strip() or None
 
     def set(self, val):
-        self.setText(unicode_type(val or ''))
+        self.setText(str(val or ''))
 
 
 class Path(QWidget):
@@ -187,14 +181,14 @@ class Path(QWidget):
         self.l = l = QHBoxLayout(self)
         l.setContentsMargins(0, 0, 0, 0)
         self.text = t = HistoryLineEdit(self)
-        t.initialize('server-opts-{}'.format(name))
+        t.initialize(f'server-opts-{name}')
         t.setClearButtonEnabled(True)
         t.currentTextChanged.connect(self.changed_signal.emit)
         l.addWidget(t)
 
         self.b = b = QToolButton(self)
         l.addWidget(b)
-        b.setIcon(QIcon(I('document_open.png')))
+        b.setIcon(QIcon.ic('document_open.png'))
         b.setToolTip(_("Browse for the file"))
         b.clicked.connect(self.choose)
         init_opt(self, opt, layout)
@@ -203,7 +197,7 @@ class Path(QWidget):
         return self.text.text().strip() or None
 
     def set(self, val):
-        self.text.setText(unicode_type(val or ''))
+        self.text.setText(str(val or ''))
 
     def choose(self):
         ans = choose_files(self, 'choose_path_srv_opts_' + self.dname, _('Choose a file'), select_only_single_file=True)
@@ -221,7 +215,7 @@ class Choices(QComboBox):
         self.setEditable(False)
         opt = options[name]
         self.choices = opt.choices
-        tuple(map(self.addItem, opt.choices))
+        self.addItems(opt.choices)
         self.currentIndexChanged.connect(self.changed_signal.emit)
         init_opt(self, opt, layout)
 
@@ -242,10 +236,10 @@ class AdvancedTab(QWidget):
     def __init__(self, parent=None):
         QWidget.__init__(self, parent)
         self.l = l = QFormLayout(self)
-        l.setFieldGrowthPolicy(l.AllNonFixedFieldsGrow)
+        l.setFieldGrowthPolicy(QFormLayout.FieldGrowthPolicy.AllNonFixedFieldsGrow)
         self.widgets = []
         self.widget_map = {}
-        self.setSizePolicy(QSizePolicy.Expanding, QSizePolicy.Fixed)
+        self.setSizePolicy(QSizePolicy.Policy.Expanding, QSizePolicy.Policy.Fixed)
         for name in sorted(options, key=lambda n: options[n].shortdoc.lower()):
             if name in ('auth', 'port', 'allow_socket_preallocation', 'userdb'):
                 continue
@@ -323,7 +317,7 @@ class MainTab(QWidget):  # {{{
         fl.addRow(options['port'].shortdoc + ':', sb)
         l.addSpacing(25)
         self.opt_auth = cb = QCheckBox(
-            _('Require &username and password to access the content server')
+            _('Require &username and password to access the Content server')
         )
         l.addWidget(cb)
         self.auth_desc = la = QLabel(self)
@@ -399,7 +393,7 @@ class MainTab(QWidget):  # {{{
 
     def change_auth_desc(self):
         self.auth_desc.setText(
-            _('Remember to create some user accounts in the "User accounts" tab')
+            _('Remember to create at least one user account in the "User accounts" tab')
             if self.opt_auth.isChecked() else _(
                 'Requiring a username/password prevents unauthorized people from'
                 ' accessing your calibre library. It is also needed for some features'
@@ -447,7 +441,7 @@ class NewUser(QDialog):
             if username else _('Add new user')
         )
         self.l = l = QFormLayout(self)
-        l.setFieldGrowthPolicy(l.AllNonFixedFieldsGrow)
+        l.setFieldGrowthPolicy(QFormLayout.FieldGrowthPolicy.AllNonFixedFieldsGrow)
         self.uw = u = QLineEdit(self)
         l.addRow(_('&Username:'), u)
         if username:
@@ -457,7 +451,7 @@ class NewUser(QDialog):
         self.p1, self.p2 = p1, p2 = QLineEdit(self), QLineEdit(self)
         l.addRow(_('&Password:'), p1), l.addRow(_('&Repeat password:'), p2)
         for p in p1, p2:
-            p.setEchoMode(QLineEdit.PasswordEchoOnEdit)
+            p.setEchoMode(QLineEdit.EchoMode.PasswordEchoOnEdit)
             p.setMinimumWidth(300)
             if username:
                 p.setText(user_data[username]['pw'])
@@ -465,17 +459,17 @@ class NewUser(QDialog):
         sp.stateChanged.connect(self.show_password)
         l.addRow(sp)
         self.bb = bb = QDialogButtonBox(
-            QDialogButtonBox.Ok | QDialogButtonBox.Cancel
+            QDialogButtonBox.StandardButton.Ok | QDialogButtonBox.StandardButton.Cancel
         )
         l.addRow(bb)
         bb.accepted.connect(self.accept), bb.rejected.connect(self.reject)
-        (self.uw if not username else self.p1).setFocus(Qt.OtherFocusReason)
+        (self.uw if not username else self.p1).setFocus(Qt.FocusReason.OtherFocusReason)
 
     def show_password(self):
         for p in self.p1, self.p2:
             p.setEchoMode(
-                QLineEdit.Normal
-                if self.showp.isChecked() else QLineEdit.PasswordEchoOnEdit
+                QLineEdit.EchoMode.Normal
+                if self.showp.isChecked() else QLineEdit.EchoMode.PasswordEchoOnEdit
             )
 
     @property
@@ -538,10 +532,10 @@ class Library(QWidget):
         self.name = name
         self.enable_on_checked = enable_on_checked
         self.l = l = QVBoxLayout(self)
-        l.setSizeConstraint(l.SetMinAndMaxSize)
+        l.setSizeConstraint(QLayout.SizeConstraint.SetMinAndMaxSize)
         if not is_first:
             self.border = b = QFrame(self)
-            b.setFrameStyle(b.HLine)
+            b.setFrameStyle(QFrame.Shape.HLine)
             l.addWidget(b)
         self.cw = cw = QCheckBox(name.replace('&', '&&'))
         cw.setStyleSheet('QCheckBox { font-weight: bold }')
@@ -590,7 +584,7 @@ class ChangeRestriction(QDialog):
         self.username = username
         self._items = []
         self.l = l = QFormLayout(self)
-        l.setFieldGrowthPolicy(l.AllNonFixedFieldsGrow)
+        l.setFieldGrowthPolicy(QFormLayout.FieldGrowthPolicy.AllNonFixedFieldsGrow)
 
         self.libraries = t = QWidget(self)
         t.setObjectName('libraries')
@@ -620,7 +614,7 @@ class ChangeRestriction(QDialog):
         self.atype_changed()
 
         self.bb = bb = QDialogButtonBox(
-            QDialogButtonBox.Ok | QDialogButtonBox.Cancel
+            QDialogButtonBox.StandardButton.Ok | QDialogButtonBox.StandardButton.Cancel
         )
         bb.accepted.connect(self.accept), bb.rejected.connect(self.reject)
         l.addWidget(bb)
@@ -703,7 +697,7 @@ class ChangeRestriction(QDialog):
             else:
                 m = _('{} is allowed access to all libraries, <b>except</b> those'
                       ' whose names match one of the names specified below.')
-                sheet += 'QWidget#libraries { background-color: #FAE7B5}'
+                sheet += f'QWidget#libraries {{ background-color: {QApplication.instance().emphasis_window_background_color} }}'
             self.libraries.setEnabled(True), self.la.setEnabled(True)
             self.items = self.items
         self.msg.setText(m.format(self.username))
@@ -716,9 +710,9 @@ class User(QWidget):
 
     def __init__(self, parent=None):
         QWidget.__init__(self, parent)
-        self.setSizePolicy(QSizePolicy.Expanding, QSizePolicy.Expanding)
+        self.setSizePolicy(QSizePolicy.Policy.Expanding, QSizePolicy.Policy.Expanding)
         self.l = l = QFormLayout(self)
-        l.setFieldGrowthPolicy(QFormLayout.AllNonFixedFieldsGrow)
+        l.setFieldGrowthPolicy(QFormLayout.FieldGrowthPolicy.AllNonFixedFieldsGrow)
         self.username_label = la = QLabel('')
         l.addWidget(la)
         self.ro_text = _('Allow {} to make &changes (i.e. grant write access)')
@@ -744,7 +738,7 @@ class User(QWidget):
 
     def change_password(self):
         d = NewUser(self.user_data, self, self.username)
-        if d.exec_() == d.Accepted:
+        if d.exec() == QDialog.DialogCode.Accepted:
             self.user_data[self.username]['pw'] = d.password
             self.changed_signal.emit()
 
@@ -801,7 +795,7 @@ class User(QWidget):
             self.user_data[self.username]['restriction'].copy(),
             parent=self
         )
-        if d.exec_() == d.Accepted:
+        if d.exec() == QDialog.DialogCode.Accepted:
             self.user_data[self.username]['restriction'] = d.restriction
             self.update_restriction()
             self.changed_signal.emit()
@@ -824,11 +818,11 @@ class Users(QWidget):
 
         self.h = h = QHBoxLayout()
         lp.addLayout(h)
-        self.add_button = b = QPushButton(QIcon(I('plus.png')), _('&Add user'), self)
+        self.add_button = b = QPushButton(QIcon.ic('plus.png'), _('&Add user'), self)
         b.clicked.connect(self.add_user)
         h.addWidget(b)
         self.remove_button = b = QPushButton(
-            QIcon(I('minus.png')), _('&Remove user'), self
+            QIcon.ic('minus.png'), _('&Remove user'), self
         )
         b.clicked.connect(self.remove_user)
         h.addStretch(2), h.addWidget(b)
@@ -836,7 +830,7 @@ class Users(QWidget):
         self.user_list = w = QListWidget(self)
         w.setSpacing(1)
         w.doubleClicked.connect(self.current_user_activated)
-        w.setSizePolicy(QSizePolicy.Preferred, QSizePolicy.Expanding)
+        w.setSizePolicy(QSizePolicy.Policy.Preferred, QSizePolicy.Policy.Expanding)
         lp.addWidget(w)
 
         self.user_display = u = User(self)
@@ -865,7 +859,7 @@ class Users(QWidget):
 
     def add_user(self):
         d = NewUser(self.user_data, parent=self)
-        if d.exec_() == d.Accepted:
+        if d.exec() == QDialog.DialogCode.Accepted:
             un, pw = d.username, d.password
             self.user_data[un] = create_user_data(pw)
             self.user_list.insertItem(0, un)
@@ -897,7 +891,7 @@ class CustomList(QWidget):  # {{{
         QWidget.__init__(self, parent)
         self.default_template = default_custom_list_template()
         self.l = l = QFormLayout(self)
-        l.setFieldGrowthPolicy(l.AllNonFixedFieldsGrow)
+        l.setFieldGrowthPolicy(QFormLayout.FieldGrowthPolicy.AllNonFixedFieldsGrow)
         self.la = la = QLabel('<p>' + _(
             'Here you can create a template to control what data is shown when'
             ' using the <i>Custom list</i> mode for the book list'))
@@ -937,9 +931,9 @@ class CustomList(QWidget):  # {{{
         l.addRow(t)
         t.textChanged.connect(self.changed_signal)
         self.imex = bb = QDialogButtonBox(self)
-        b = bb.addButton(_('&Import template'), bb.ActionRole)
+        b = bb.addButton(_('&Import template'), QDialogButtonBox.ButtonRole.ActionRole)
         b.clicked.connect(self.import_template)
-        b = bb.addButton(_('E&xport template'), bb.ActionRole)
+        b = bb.addButton(_('E&xport template'), QDialogButtonBox.ButtonRole.ActionRole)
         b.clicked.connect(self.export_template)
         l.addRow(bb)
 
@@ -947,7 +941,7 @@ class CustomList(QWidget):  # {{{
         paths = choose_files(self, 'custom-list-template', _('Choose template file'),
             filters=[(_('Template files'), ['json'])], all_files=False, select_only_single_file=True)
         if paths:
-            with lopen(paths[0], 'rb') as f:
+            with open(paths[0], 'rb') as f:
                 raw = f.read()
             self.current_template = self.deserialize(raw)
 
@@ -957,7 +951,7 @@ class CustomList(QWidget):  # {{{
             filters=[(_('Template files'), ['json'])], initial_filename='custom-list-template.json')
         if path:
             raw = self.serialize(self.current_template)
-            with lopen(path, 'wb') as f:
+            with open(path, 'wb') as f:
                 f.write(as_bytes(raw))
 
     def thumbnail_state_changed(self):
@@ -1005,12 +999,12 @@ class CustomList(QWidget):  # {{{
         if template == self.default_template:
             try:
                 os.remove(custom_list_template.path)
-            except EnvironmentError as err:
+            except OSError as err:
                 if err.errno != errno.ENOENT:
                     raise
         else:
             raw = self.serialize(template)
-            with lopen(custom_list_template.path, 'wb') as f:
+            with open(custom_list_template.path, 'wb') as f:
                 f.write(as_bytes(raw))
         return True
 
@@ -1028,7 +1022,7 @@ class URLItem(QWidget):
         self.changed_signal.connect(parent.changed_signal)
         self.l = l = QFormLayout(self)
         self.type_widget = t = QComboBox(self)
-        l.setFieldGrowthPolicy(l.ExpandingFieldsGrow)
+        l.setFieldGrowthPolicy(QFormLayout.FieldGrowthPolicy.ExpandingFieldsGrow)
         t.addItems([_('Book'), _('Author')])
         l.addRow(_('URL type:'), t)
         self.name_widget = n = QLineEdit(self)
@@ -1112,7 +1106,7 @@ class SearchTheInternet(QWidget):
 
         self.h = QHBoxLayout()
         gl.addLayout(self.h)
-        self.add_url_button = b = QPushButton(QIcon(I('plus.png')), _('&Add URL'))
+        self.add_url_button = b = QPushButton(QIcon.ic('plus.png'), _('&Add URL'))
         b.clicked.connect(self.add_url)
         self.h.addWidget(b)
         self.export_button = b = QPushButton(_('Export URLs'))
@@ -1160,7 +1154,7 @@ class SearchTheInternet(QWidget):
         sb = self.sa.verticalScrollBar()
         if sb:
             sb.setValue(sb.maximum())
-        self.items[-1].name_widget.setFocus(Qt.OtherFocusReason)
+        self.items[-1].name_widget.setFocus(Qt.FocusReason.OtherFocusReason)
 
     @property
     def serialized_urls(self):
@@ -1172,12 +1166,12 @@ class SearchTheInternet(QWidget):
                 return False
         cu = self.current_urls
         if cu:
-            with lopen(search_the_net_urls.path, 'wb') as f:
-                f.write(self.serialized_urls)
+            with open(search_the_net_urls.path, 'wb') as f:
+                f.write(self.serialized_urls.encode('utf-8'))
         else:
             try:
                 os.remove(search_the_net_urls.path)
-            except EnvironmentError as err:
+            except OSError as err:
                 if err.errno != errno.ENOENT:
                     raise
         return True
@@ -1187,14 +1181,14 @@ class SearchTheInternet(QWidget):
             self, 'search-net-urls', _('Choose URLs file'),
             filters=[(_('URL files'), ['json'])], initial_filename='search-urls.json')
         if path:
-            with lopen(path, 'wb') as f:
-                f.write(self.serialized_urls)
+            with open(path, 'wb') as f:
+                f.write(self.serialized_urls.encode('utf-8'))
 
     def import_urls(self):
         paths = choose_files(self, 'search-net-urls', _('Choose URLs file'),
             filters=[(_('URL files'), ['json'])], all_files=False, select_only_single_file=True)
         if paths:
-            with lopen(paths[0], 'rb') as f:
+            with open(paths[0], 'rb') as f:
                 items = json.loads(f.read())
                 [self.append_item(x) for x in items]
                 self.changed_signal.emit()
@@ -1267,7 +1261,7 @@ class ConfigWidget(ConfigWidgetBase):
     def start_server(self):
         if not self.save_changes():
             return
-        self.setCursor(Qt.BusyCursor)
+        self.setCursor(Qt.CursorShape.BusyCursor)
         try:
             self.gui.start_content_server(check_started=False)
             while (not self.server.is_running and self.server.exception is None):
@@ -1277,7 +1271,7 @@ class ConfigWidget(ConfigWidgetBase):
                     self,
                     _('Failed to start Content server'),
                     as_unicode(self.gui.content_server.exception)
-                ).exec_()
+                ).exec()
                 self.gui.content_server = None
                 return
             self.main_tab.update_button_state()
@@ -1293,7 +1287,7 @@ class ConfigWidget(ConfigWidgetBase):
             show_copy_button=False
         )
         QTimer.singleShot(500, self.check_exited)
-        self.stopping_msg.exec_()
+        self.stopping_msg.exec()
 
     def check_exited(self):
         if getattr(self.server, 'is_running', False):
@@ -1305,13 +1299,12 @@ class ConfigWidget(ConfigWidgetBase):
         self.stopping_msg.accept()
 
     def test_server(self):
+        from calibre.utils.network import format_addr_for_url, get_fallback_server_addr
         prefix = self.advanced_tab.get('url_prefix') or ''
         protocol = 'https' if self.advanced_tab.has_ssl else 'http'
-        lo = self.advanced_tab.get('listen_on') or '0.0.0.0'
-        lo = {'0.0.0.0': '127.0.0.1', '::':'::1'}.get(lo)
-        url = '{protocol}://{interface}:{port}{prefix}'.format(
-            protocol=protocol, interface=lo,
-            port=self.main_tab.opt_port.value(), prefix=prefix)
+        addr = self.advanced_tab.get('listen_on') or get_fallback_server_addr()
+        addr = {'0.0.0.0': '127.0.0.1', '::': '::1'}.get(addr, addr)
+        url = f'{protocol}://{format_addr_for_url(addr)}:{self.main_tab.opt_port.value()}{prefix}'
         open_url(QUrl(url))
 
     def view_server_logs(self):
@@ -1328,7 +1321,7 @@ class ConfigWidget(ConfigWidgetBase):
             el.setPlainText(
                 share_open(log_error_file, 'rb').read().decode('utf8', 'replace')
             )
-        except EnvironmentError:
+        except OSError:
             el.setPlainText(_('No error log found'))
         layout.addWidget(QLabel(_('Access log:')))
         al = QPlainTextEdit(d)
@@ -1337,15 +1330,15 @@ class ConfigWidget(ConfigWidgetBase):
             al.setPlainText(
                 share_open(log_access_file, 'rb').read().decode('utf8', 'replace')
             )
-        except EnvironmentError:
+        except OSError:
             al.setPlainText(_('No access log found'))
         loc = QLabel(_('The server log files are in: {}').format(os.path.dirname(log_error_file)))
         loc.setWordWrap(True)
         layout.addWidget(loc)
-        bx = QDialogButtonBox(QDialogButtonBox.Ok)
+        bx = QDialogButtonBox(QDialogButtonBox.StandardButton.Ok)
         layout.addWidget(bx)
         bx.accepted.connect(d.accept)
-        b = bx.addButton(_('&Clear logs'), bx.ActionRole)
+        b = bx.addButton(_('&Clear logs'), QDialogButtonBox.ButtonRole.ActionRole)
 
         def clear_logs():
             if getattr(self.server, 'is_running', False):
@@ -1358,7 +1351,7 @@ class ConfigWidget(ConfigWidgetBase):
                 for x in (log_error_file, log_access_file):
                     try:
                         os.remove(x)
-                    except EnvironmentError as err:
+                    except OSError as err:
                         if err.errno != errno.ENOENT:
                             raise
             el.setPlainText(''), al.setPlainText('')
@@ -1378,7 +1371,7 @@ class ConfigWidget(ConfigWidgetBase):
                     _('No users specified'),
                     _(
                         'You have turned on the setting to require passwords to access'
-                        ' the content server, but you have not created any user accounts.'
+                        ' the Content server, but you have not created any user accounts.'
                         ' Create at least one user account in the "User accounts" tab to proceed.'
                     ),
                     show=True
