@@ -707,6 +707,12 @@ class DateTimeEdit(QDateTimeEdit):
         self.setMinimumDateTime(UNDEFINED_QDATETIME)
         self.setCalendarPopup(True)
         self.cw = CalendarWidget(self)
+        if tweaks['calendar_start_day_of_week'] != 'Default':
+            try:
+                dow = Qt.DayOfWeek[tweaks['calendar_start_day_of_week']]
+                self.cw.setFirstDayOfWeek(dow)
+            except Exception:
+                print(f"Bad value for tweak calendar_start_day_of_week: {tweaks['calendar_start_day_of_week']}")
         self.cw.setVerticalHeaderFormat(QCalendarWidget.VerticalHeaderFormat.NoVerticalHeader)
         self.setCalendarWidget(self.cw)
         self.setSpecialValueText(_('Undefined'))
@@ -822,7 +828,7 @@ class MessagePopup(QLabel):
         self.resize(self.sizeHint())
         self.position_in_parent()
         self.show()
-        self.raise_()
+        self.raise_without_focus()
         self.close_timer.start(timeout)
 
     def position_in_parent(self):
