@@ -1,25 +1,27 @@
 #!/usr/bin/env python
-# vim:fileencoding=UTF-8:ts=4:sw=4:sta:et:sts=4:ai
 
 
 __license__   = 'GPL v3'
 __copyright__ = '2009, Kovid Goyal <kovid@kovidgoyal.net>'
 __docformat__ = 'restructuredtext en'
 
-import os, traceback, re, errno
+import os
+import re
+import traceback
 
-from calibre.constants import DEBUG
+from calibre import prints, sanitize_file_name, strftime
+from calibre.constants import DEBUG, iswindows, preferred_encoding
 from calibre.db.errors import NoSuchFormat
-from calibre.utils.config import Config, StringConfig, tweaks
-from calibre.utils.formatter import TemplateFormatter
-from calibre.utils.filenames import shorten_components_to, ascii_filename
-from calibre.constants import preferred_encoding
-from calibre.ebooks.metadata import fmt_sidx
-from calibre.ebooks.metadata import title_sort
-from calibre.utils.date import as_local_time
-from calibre import strftime, prints, sanitize_file_name
 from calibre.db.lazy import FormatsList
-from polyglot.builtins import unicode_type
+from calibre.ebooks.metadata import fmt_sidx, title_sort
+from calibre.utils.config import Config, StringConfig, tweaks
+from calibre.utils.date import as_local_time, is_date_undefined
+from calibre.utils.filenames import (
+    ascii_filename, make_long_path_useable, shorten_components_to,
+)
+from calibre.utils.formatter import TemplateFormatter
+from calibre.utils.formatter_functions import load_user_template_functions
+from calibre.utils.localization import _
 
 plugboard_any_device_value = 'any device'
 plugboard_any_format_value = 'any format'
@@ -94,16 +96,16 @@ def config(defaults=None):
             help=_('Comma separated list of formats to save for each book.'
                 ' By default all available formats are saved.'))
     x('template', default=DEFAULT_TEMPLATE,
-            help=_('The template to control the filename and directory structure of the saved files. '
+            help=_('The template to control the filename and folder structure of the saved files. '
                 'Default is "%(templ)s" which will save books into a per-author '
-                'subdirectory with filenames containing title and author. '
+                'subfolder with filenames containing title and author. '
                 'Available controls are: {%(controls)s}')%dict(
                     templ=DEFAULT_TEMPLATE, controls=', '.join(sorted(FORMAT_ARGS))))
     x('send_template', default=DEFAULT_SEND_TEMPLATE,
-            help=_('The template to control the filename and directory structure of files '
+            help=_('The template to control the filename and folder structure of files '
                 'sent to the device. '
                 'Default is "%(templ)s" which will save books into a per-author '
-                'directory with filenames containing title and author. '
+                'folder with filenames containing title and author. '
                 'Available controls are: {%(controls)s}')%dict(
                     templ=DEFAULT_SEND_TEMPLATE, controls=', '.join(FORMAT_ARGS)))
     x('asciiize', default=False,
@@ -123,8 +125,10 @@ def config(defaults=None):
     x('replace_whitespace', default=False,
             help=_('Replace whitespace with underscores.'))
     x('single_dir', default=False,
-            help=_('Save into a single directory, ignoring the template'
-                ' directory structure'))
+            help=_('Save into a single folder, ignoring the template'
+                ' folder structure'))
+    x('save_extra_files', default=True, help=_(
+        'Save any data files associated with the book when saving the book'))
     return c
 
 
@@ -132,7 +136,7 @@ def preprocess_template(template):
     template = template.replace('//', '/')
     template = template.replace('{author}', '{authors}')
     template = template.replace('{tag}', '{tags}')
-    if not isinstance(template, unicode_type):
+    if not isinstance(template, str):
         template = template.decode(preferred_encoding, 'replace')
     return template
 
@@ -143,7 +147,7 @@ class Formatter(TemplateFormatter):
     '''
 
     def get_value(self, key, args, kwargs):
-        if key == '':
+        if not isinstance(key, str) or key == '':
             return ''
         try:
             key = key.lower()
@@ -157,7 +161,7 @@ class Formatter(TemplateFormatter):
                     return self.composite_values[key]
                 self.composite_values[key] = 'RECURSIVE_COMPOSITE FIELD (S2D) ' + key
                 self.composite_values[key] = \
-                    self.evaluate(b['display']['composite_template'], [], kwargs)
+                    self.evaluate(b['display']['composite_template'], [], kwargs, {})
                 return self.composite_values[key]
             if key in kwargs:
                 val = kwargs[key]
@@ -208,14 +212,15 @@ def get_components(template, mi, id, timefmt='%b %Y', length=250,
     else:
         format_args['identifiers'] = ''
 
-    if hasattr(mi.timestamp, 'timetuple'):
+    if not is_date_undefined(mi.timestamp) and hasattr(mi.timestamp, 'timetuple'):
         format_args['timestamp'] = strftime(timefmt, mi.timestamp.timetuple())
-    if hasattr(mi.pubdate, 'timetuple'):
+    if not is_date_undefined(mi.pubdate) and hasattr(mi.pubdate, 'timetuple'):
         format_args['pubdate'] = strftime(timefmt, mi.pubdate.timetuple())
-    if hasattr(mi, 'last_modified') and hasattr(mi.last_modified, 'timetuple'):
+    if (hasattr(mi, 'last_modified') and not is_date_undefined(mi.last_modified) and
+                hasattr(mi.last_modified, 'timetuple')):
         format_args['last_modified'] = strftime(timefmt, mi.last_modified.timetuple())
 
-    format_args['id'] = unicode_type(id)
+    format_args['id'] = str(id)
     # Now format the custom fields
     custom_metadata = mi.get_all_user_metadata(make_copy=False)
     for key in custom_metadata:
@@ -234,7 +239,7 @@ def get_components(template, mi, id, timefmt='%b %Y', length=250,
                         divide_by=2.0)
             elif cm['datatype'] in ['int', 'float']:
                 if format_args[key] != 0:
-                    format_args[key] = unicode_type(format_args[key])
+                    format_args[key] = str(format_args[key])
                 else:
                     format_args[key] = ''
     if safe_format:
@@ -245,7 +250,7 @@ def get_components(template, mi, id, timefmt='%b %Y', length=250,
     components = [x.strip() for x in components.split('/')]
     components = [sanitize_func(x) for x in components if x]
     if not components:
-        components = [unicode_type(id)]
+        components = [str(id)]
     if to_lowercase:
         components = [x.lower() for x in components]
     if replace_whitespace:
@@ -319,7 +324,7 @@ def update_metadata(mi, fmt, stream, plugboards, cdata, error_report=None, plugb
 
 
 def do_save_book_to_disk(db, book_id, mi, plugboards,
-        formats, root, opts, length):
+        formats, root, opts, length, extra_files=()):
     originals = mi.cover, mi.pubdate, mi.timestamp
     formats_written = False
     try:
@@ -332,40 +337,43 @@ def do_save_book_to_disk(db, book_id, mi, plugboards,
         base_path = os.path.join(root, *components)
         base_name = os.path.basename(base_path)
         dirpath = os.path.dirname(base_path)
-        try:
-            os.makedirs(dirpath)
-        except EnvironmentError as err:
-            if err.errno != errno.EEXIST:
-                raise
-
+        os.makedirs(dirpath, exist_ok=True)
         cdata = None
         if opts.save_cover:
             cdata = db.cover(book_id)
             if cdata:
                 cpath = base_path + '.jpg'
-                with lopen(cpath, 'wb') as f:
+                with open(make_long_path_useable(cpath), 'wb') as f:
                     f.write(cdata)
                 mi.cover = base_name+'.jpg'
         if opts.write_opf:
             from calibre.ebooks.metadata.opf2 import metadata_to_opf
             opf = metadata_to_opf(mi)
-            with lopen(base_path+'.opf', 'wb') as f:
+            with open(make_long_path_useable(base_path+'.opf'), 'wb') as f:
                 f.write(opf)
     finally:
         mi.cover, mi.pubdate, mi.timestamp = originals
 
+    if extra_files and opts.save_extra_files:
+        for relpath in extra_files:
+            data_dest_path = os.path.abspath(os.path.join(dirpath, relpath))
+            try:
+                db.copy_extra_file_to(book_id, relpath, data_dest_path)
+            except FileNotFoundError:
+                os.makedirs(make_long_path_useable(os.path.dirname(data_dest_path)))
+                db.copy_extra_file_to(book_id, relpath, data_dest_path)
     if not formats:
         return not formats_written, book_id, mi.title
 
     for fmt in formats:
-        fmt_path = base_path+'.'+unicode_type(fmt)
+        fmt_path = base_path+'.'+str(fmt)
         try:
             db.copy_format_to(book_id, fmt, fmt_path)
             formats_written = True
         except NoSuchFormat:
             continue
         if opts.update_metadata:
-            with lopen(fmt_path, 'r+b') as stream:
+            with open(make_long_path_useable(fmt_path), 'r+b') as stream:
                 update_metadata(mi, fmt, stream, plugboards, cdata)
 
     return not formats_written, book_id, mi.title
@@ -377,7 +385,8 @@ def sanitize_args(root, opts):
     root = os.path.abspath(root)
 
     opts.template = preprocess_template(opts.template)
-    length = 240
+    length = 255 if iswindows else 1024
+    length -= 15
     length -= len(root)
     if length < 5:
         raise ValueError('%r is too long.'%root)
@@ -425,14 +434,16 @@ def read_serialized_metadata(data):
     mi.cover, mi.cover_data = None, (None, None)
     cdata = None
     if 'cover' in data:
-        with lopen(data['cover'], 'rb') as f:
+        with open(data['cover'], 'rb') as f:
             cdata = f.read()
     return mi, cdata
 
 
 def update_serialized_metadata(book, common_data=None):
+    # This is called from a worker process. It must not open the database.
     result = []
-    plugboard_cache = common_data
+    plugboard_cache = common_data['plugboard_cache']
+    load_user_template_functions(common_data['library_id'], common_data['template_functions'])
     from calibre.customize.ui import apply_null_metadata
     with apply_null_metadata:
         fmts = [fp.rpartition(os.extsep)[-1] for fp in book['fmts']]
@@ -443,7 +454,7 @@ def update_serialized_metadata(book, common_data=None):
 
         for fmt, fmtpath in zip(fmts, book['fmts']):
             try:
-                with lopen(fmtpath, 'r+b') as stream:
+                with open(fmtpath, 'r+b') as stream:
                     update_metadata(mi, fmt, stream, (), cdata, error_report=report_error, plugboard_cache=plugboard_cache)
             except Exception:
                 report_error(fmt, traceback.format_exc())
