@@ -30,7 +30,9 @@ from calibre.srv.routes import endpoint, json
 from calibre.srv.utils import get_db, get_use_roman, http_date
 from calibre.utils.config_base import tweaks
 from calibre.utils.date import timestampfromdt
-from calibre.utils.filenames import ascii_filename, atomic_rename
+from calibre.utils.filenames import (
+    ascii_filename, atomic_rename, make_long_path_useable,
+)
 from calibre.utils.img import image_from_data, scale_image
 from calibre.utils.localization import _
 from calibre.utils.resources import get_image_path as I, get_path as P
@@ -303,6 +305,45 @@ def icon(ctx, rd, which):
         return ans
 
 
+@endpoint('/reader-background/{encoded_fname}', android_workaround=True)
+def reader_background(ctx, rd, encoded_fname):
+    base = os.path.abspath(os.path.normapth(os.path.join(config_dir, 'viewer', 'background-images')))
+    fname = bytes.fromhex(encoded_fname)
+    q = os.path.abspath(os.path.normpath(os.path.join(base, fname)))
+    if not q.startswith(base):
+        raise HTTPNotFound(f'Reader background {encoded_fname} not found')
+    try:
+        return share_open(make_long_path_useable(q), 'rb')
+    except FileNotFoundError:
+        raise HTTPNotFound(f'Reader background {encoded_fname} not found')
+
+
+@endpoint('/reader-profiles/get-all', postprocess=json)
+def get_all_reader_profiles(ctx, rd):
+    from calibre.gui2.viewer.config import load_viewer_profiles
+    which = 'user:'
+    if rd.username:
+        which += rd.username
+    return load_viewer_profiles(which)
+
+
+@endpoint('/reader-profiles/save', methods={'POST'}, postprocess=json)
+def save_reader_profile(ctx, rd):
+    try:
+        data = load_json_file(rd.request_body_file)
+        name, profile = data['name'], data['profile']
+        if not isinstance(profile, dict) and profile is not None:
+            raise TypeError(f'profile must be a dict not {type(profile)}')
+    except Exception as err:
+        raise HTTPBadRequest(f'Invalid query: {err}')
+    from calibre.gui2.viewer.config import save_viewer_profile
+    which = 'user:'
+    if rd.username:
+        which += rd.username
+    save_viewer_profile(name, profile, which)
+    return True
+
+
 @endpoint('/get/{what}/{book_id}/{library_id=None}', android_workaround=True)
 def get(ctx, rd, what, book_id, library_id):
     book_id, rest = book_id.partition('_')[::2]
