@@ -1,4 +1,3 @@
-from __future__ import absolute_import, division, print_function, unicode_literals
 '''
 Make strings safe for use as ASCII filenames, while trying to preserve as much
 meaning as possible.
@@ -8,14 +7,15 @@ import errno
 import os
 import shutil
 import time
+from contextlib import closing, suppress
 from math import ceil
 
 from calibre import force_unicode, isbytestring, prints, sanitize_file_name
 from calibre.constants import (
-    filesystem_encoding, iswindows, plugins, preferred_encoding, isosx, ispy3
+    filesystem_encoding, ismacos, iswindows, preferred_encoding,
 )
-from calibre.utils.localization import get_udc
-from polyglot.builtins import iteritems, itervalues, unicode_type, range
+from calibre.utils.localization import _, get_udc
+from polyglot.builtins import iteritems, itervalues
 
 
 def ascii_text(orig):
@@ -23,7 +23,7 @@ def ascii_text(orig):
     try:
         ascii = udc.decode(orig)
     except Exception:
-        if isinstance(orig, unicode_type):
+        if isinstance(orig, str):
             orig = orig.encode('ascii', 'replace')
         ascii = orig.decode(preferred_encoding, 'replace')
     if isinstance(ascii, bytes):
@@ -52,7 +52,7 @@ def shorten_component(s, by_what):
 def limit_component(x, limit=254):
     # windows and macs use ytf-16 codepoints for length, linux uses arbitrary
     # binary data, but we will assume utf-8
-    filename_encoding_for_length = 'utf-16' if iswindows or isosx else 'utf-8'
+    filename_encoding_for_length = 'utf-16' if iswindows or ismacos else 'utf-8'
 
     def encoded_length():
         q = x if isinstance(x, bytes) else x.encode(filename_encoding_for_length)
@@ -125,7 +125,7 @@ def is_case_sensitive(path):
         name1, name2 = ('calibre_test_case_sensitivity.txt',
                         'calibre_TesT_CaSe_sensitiVitY.Txt')
         f1, f2 = os.path.join(path, name1), os.path.join(path, name2)
-        if os.path.exists(f1):
+        with suppress(OSError):
             os.remove(f1)
         open(f1, 'w').close()
         is_case_sensitive = not os.path.exists(f2)
@@ -133,6 +133,47 @@ def is_case_sensitive(path):
     return is_case_sensitive
 
 
+def case_ignoring_open_file(path, mode='r'):
+    '''
+    Open an existing file case insensitively, even on case sensitive file systems
+    '''
+    try:
+        return open(path, mode)
+    except FileNotFoundError as err:
+        original_err = err
+
+    def next_component(final_path, components):
+        if not components:
+            return final_path
+        component = components.pop()
+        cl = component.lower()
+        try:
+            matches = {x for x in os.listdir(final_path) if x.lower() == cl}
+        except OSError:
+            raise original_err from None
+        for x in matches:
+            current = os.path.join(final_path, x)
+            try:
+                return next_component(current, list(components))
+            except Exception:
+                continue
+        raise original_err
+
+    if isbytestring(path):
+        path = path.decode(filesystem_encoding)
+    if path.endswith(os.sep):
+        path = path[:-1]
+    if not path:
+        raise ValueError('Path must not point to root')
+
+    components = path.split(os.sep)
+    if len(components) <= 1:
+        raise ValueError(f'Invalid path: {path}')
+    final_path = (components[0].upper() + os.sep) if iswindows else '/'
+    components = list(reversed(components))[:-1]
+    return open(next_component(final_path, components), mode)
+
+
 def case_preserving_open_file(path, mode='wb', mkdir_mode=0o777):
     '''
     Open the file pointed to by path with the specified mode. If any
@@ -197,7 +238,7 @@ def case_preserving_open_file(path, mode='wb', mkdir_mode=0o777):
         ans = fpath = cpath
     else:
         fname = components[-1]
-        ans = lopen(os.path.join(cpath, fname), mode)
+        ans = open(os.path.join(cpath, fname), mode)
         # Ensure file and all its metadata is written to disk so that subsequent
         # listdir() has file name in it. I don't know if this is actually
         # necessary, but given the diversity of platforms, best to be safe.
@@ -207,7 +248,7 @@ def case_preserving_open_file(path, mode='wb', mkdir_mode=0o777):
         cl = fname.lower()
         try:
             candidates = [c for c in os.listdir(cpath) if c.lower() == cl]
-        except EnvironmentError:
+        except OSError:
             # The containing directory, somehow disappeared?
             candidates = []
         if len(candidates) == 1:
@@ -221,20 +262,11 @@ def case_preserving_open_file(path, mode='wb', mkdir_mode=0o777):
 def windows_get_fileid(path):
     ''' The fileid uniquely identifies actual file contents (it is the same for
     all hardlinks to a file). Similar to inode number on linux. '''
-    import win32file
-    from pywintypes import error
+    from calibre_extensions.winutil import get_file_id
     if isbytestring(path):
         path = path.decode(filesystem_encoding)
-    try:
-        h = win32file.CreateFileW(path, 0, 0, None, win32file.OPEN_EXISTING,
-                win32file.FILE_FLAG_BACKUP_SEMANTICS, 0)
-        try:
-            data = win32file.GetFileInformationByHandle(h)
-        finally:
-            win32file.CloseHandle(h)
-    except (error, EnvironmentError):
-        return None
-    return data[4], data[8], data[9]
+    with suppress(OSError):
+        return get_file_id(path)
 
 
 def samefile_windows(src, dst):
@@ -267,7 +299,7 @@ def samefile(src, dst):
         # Unix
         try:
             return os.path.samefile(src, dst)
-        except EnvironmentError:
+        except OSError:
             return False
 
     # All other platforms: check for same pathname.
@@ -280,25 +312,19 @@ def windows_get_size(path):
     ''' On windows file sizes are only accurately stored in the actual file,
     not in the directory entry (which could be out of date). So we open the
     file, and get the actual size. '''
-    import win32file
+    from calibre_extensions import winutil
     if isbytestring(path):
         path = path.decode(filesystem_encoding)
-    h = win32file.CreateFileW(
-        path, 0, win32file.FILE_SHARE_READ | win32file.FILE_SHARE_WRITE | win32file.FILE_SHARE_DELETE,
-        None, win32file.OPEN_EXISTING, 0, None)
-    try:
-        return win32file.GetFileSize(h)
-    finally:
-        win32file.CloseHandle(h)
+    with closing(winutil.create_file(
+        path, 0, winutil.FILE_SHARE_READ | winutil.FILE_SHARE_WRITE | winutil.FILE_SHARE_DELETE,
+        winutil.OPEN_EXISTING, 0)
+    ) as h:
+        return winutil.get_file_size(h)
 
 
 def windows_hardlink(src, dest):
-    import win32file, pywintypes
-    try:
-        win32file.CreateHardLink(dest, src)
-    except pywintypes.error as e:
-        msg = 'Creating hardlink from %s to %s failed: %%s' % (src, dest)
-        raise OSError(msg % e)
+    from calibre_extensions import winutil
+    winutil.create_hard_link(dest, src)
     src_size = os.path.getsize(src)
     # We open and close dest, to ensure its directory entry is updated
     # see http://blogs.msdn.com/b/oldnewthing/archive/2011/12/26/10251026.aspx
@@ -308,42 +334,33 @@ def windows_hardlink(src, dest):
         try:
             if windows_get_size(dest) == src_size:
                 return
-        except EnvironmentError:
+        except OSError:
             pass
         time.sleep(0.3)
 
     sz = windows_get_size(dest)
     if sz != src_size:
-        msg = 'Creating hardlink from %s to %s failed: %%s' % (src, dest)
+        msg = f'Creating hardlink from {src} to {dest} failed: %s'
         raise OSError(msg % ('hardlink size: %d not the same as source size' % sz))
 
 
 def windows_fast_hardlink(src, dest):
-    import win32file, pywintypes
-    try:
-        win32file.CreateHardLink(dest, src)
-    except pywintypes.error as e:
-        msg = 'Creating hardlink from %s to %s failed: %%s' % (src, dest)
-        raise OSError(msg % e)
+    from calibre_extensions import winutil
+    winutil.create_hard_link(dest, src)
     ssz, dsz = windows_get_size(src), windows_get_size(dest)
     if ssz != dsz:
-        msg = 'Creating hardlink from %s to %s failed: %%s' % (src, dest)
+        msg = f'Creating hardlink from {src} to {dest} failed: %s'
         raise OSError(msg % ('hardlink size: %d not the same as source size: %s' % (dsz, ssz)))
 
 
 def windows_nlinks(path):
-    import win32file
-    dwFlagsAndAttributes = win32file.FILE_FLAG_BACKUP_SEMANTICS if os.path.isdir(path) else 0
+    from calibre_extensions import winutil
     if isbytestring(path):
         path = path.decode(filesystem_encoding)
-    handle = win32file.CreateFileW(path, win32file.GENERIC_READ, win32file.FILE_SHARE_READ, None, win32file.OPEN_EXISTING, dwFlagsAndAttributes, None)
-    try:
-        return win32file.GetFileInformationByHandle(handle)[7]
-    finally:
-        handle.Close()
+    return winutil.nlinks(path)
 
 
-class WindowsAtomicFolderMove(object):
+class WindowsAtomicFolderMove:
 
     '''
     Move all the files inside a specified folder in an atomic fashion,
@@ -354,12 +371,11 @@ class WindowsAtomicFolderMove(object):
     '''
 
     def __init__(self, path):
-        self.handle_map = {}
-
-        import win32file, winerror
-        from pywintypes import error
         from collections import defaultdict
 
+        from calibre_extensions import winutil
+        self.handle_map = {}
+
         if isbytestring(path):
             path = path.decode(filesystem_encoding)
 
@@ -376,18 +392,16 @@ class WindowsAtomicFolderMove(object):
             f = os.path.normcase(os.path.abspath(os.path.join(path, x)))
             if not os.path.isfile(f):
                 continue
-            try:
+            with suppress(OSError):
                 # Ensure the file is not read-only
-                win32file.SetFileAttributes(f, win32file.FILE_ATTRIBUTE_NORMAL)
-            except:
-                pass
+                winutil.set_file_attributes(f, winutil.FILE_ATTRIBUTE_NORMAL)
 
             try:
-                h = win32file.CreateFileW(f, win32file.GENERIC_READ,
-                        win32file.FILE_SHARE_DELETE, None,
-                        win32file.OPEN_EXISTING, win32file.FILE_FLAG_SEQUENTIAL_SCAN, 0)
-            except error as e:
-                if getattr(e, 'winerror', 0) == winerror.ERROR_SHARING_VIOLATION:
+                h = winutil.create_file(f, winutil.GENERIC_READ,
+                        winutil.FILE_SHARE_DELETE,
+                        winutil.OPEN_EXISTING, winutil.FILE_FLAG_SEQUENTIAL_SCAN)
+            except OSError as e:
+                if e.winerror == winutil.ERROR_SHARING_VIOLATION:
                     # The file could be a hardlink to an already opened file,
                     # in which case we use the same handle for both files
                     fileid = name_to_fileid[x]
@@ -403,7 +417,7 @@ class WindowsAtomicFolderMove(object):
                         continue
 
                 self.close_handles()
-                if getattr(e, 'winerror', 0) == winerror.ERROR_SHARING_VIOLATION:
+                if e.winerror == winutil.ERROR_SHARING_VIOLATION:
                     err = IOError(errno.EACCES,
                             _('File is open in another process'))
                     err.filename = f
@@ -417,9 +431,9 @@ class WindowsAtomicFolderMove(object):
             self.handle_map[f] = h
 
     def copy_path_to(self, path, dest):
-        import win32file
+        from calibre_extensions import winutil
         handle = None
-        for p, h in iteritems(self.handle_map):
+        for p, h in self.handle_map.items():
             if samefile_windows(path, p):
                 handle = h
                 break
@@ -429,18 +443,16 @@ class WindowsAtomicFolderMove(object):
                         ' operation was started'%path)
             else:
                 raise ValueError('The file %r does not exist'%path)
-        try:
+
+        with suppress(OSError):
             windows_hardlink(path, dest)
             return
-        except:
-            pass
 
-        win32file.SetFilePointer(handle, 0, win32file.FILE_BEGIN)
-        with lopen(dest, 'wb') as f:
+        winutil.set_file_pointer(handle, 0, winutil.FILE_BEGIN)
+        with open(dest, 'wb') as f:
+            sz = 1024 * 1024
             while True:
-                hr, raw = win32file.ReadFile(handle, 1024*1024)
-                if hr != 0:
-                    raise IOError(hr, 'Error while reading from %r'%path)
+                raw = winutil.read_file(handle, sz)
                 if not raw:
                     break
                 f.write(raw)
@@ -453,26 +465,25 @@ class WindowsAtomicFolderMove(object):
                 key = (p, h)
                 break
         if key is not None:
-            import win32file
-            win32file.CloseHandle(key[1])
+            key[1].close()
             remove = [f for f, h in iteritems(self.handle_map) if h is key[1]]
             for x in remove:
                 self.handle_map.pop(x)
 
     def close_handles(self):
-        import win32file
         for h in itervalues(self.handle_map):
-            win32file.CloseHandle(h)
+            h.close()
         self.handle_map = {}
 
     def delete_originals(self):
-        import win32file
+        from calibre_extensions import winutil
         for path in self.handle_map:
-            win32file.DeleteFile(path)
+            winutil.delete_file(path)
         self.close_handles()
 
 
 def hardlink_file(src, dest):
+    src, dest = make_long_path_useable(src), make_long_path_useable(dest)
     if iswindows:
         windows_hardlink(src, dest)
         return
@@ -487,30 +498,35 @@ def nlinks_file(path):
 
 
 if iswindows:
+    from calibre_extensions.winutil import move_file
+
     def rename_file(a, b):
-        move_file = plugins['winutil'][0].move_file
         if isinstance(a, bytes):
-            a = a.decode('mbcs')
+            a = os.fsdecode(a)
         if isinstance(b, bytes):
-            b = b.decode('mbcs')
+            b = os.fsdecode(b)
         move_file(a, b)
 
 
+def retry_on_fail(func, *args, count=10, sleep_time=0.2):
+    for i in range(count):
+        try:
+            func(*args)
+            break
+        except OSError:
+            if i > count - 2:
+                raise
+            # Try the operation repeatedly in case something like a virus
+            # scanner has opened one of the files (I love windows)
+            time.sleep(sleep_time)
+
+
 def atomic_rename(oldpath, newpath):
     '''Replace the file newpath with the file oldpath. Can fail if the files
     are on different volumes. If succeeds, guaranteed to be atomic. newpath may
     or may not exist. If it exists, it is replaced. '''
     if iswindows:
-        for i in range(10):
-            try:
-                rename_file(oldpath, newpath)
-                break
-            except Exception:
-                if i > 8:
-                    raise
-                # Try the rename repeatedly in case something like a virus
-                # scanner has opened one of the files (I love windows)
-                time.sleep(1)
+        retry_on_fail(rename_file, oldpath, newpath)
     else:
         os.rename(oldpath, newpath)
 
@@ -545,20 +561,7 @@ def remove_dir_if_empty(path, ignore_metadata_caches=False):
         raise
 
 
-if iswindows and not ispy3:
-    # Python's expanduser is broken for non-ASCII usernames
-    def expanduser(path):
-        if isinstance(path, bytes):
-            path = path.decode(filesystem_encoding)
-        if path[:1] != '~':
-            return path
-        i, n = 1, len(path)
-        while i < n and path[i] not in '/\\':
-            i += 1
-        userhome = plugins['winutil'][0].special_folder_path(plugins['winutil'][0].CSIDL_PROFILE)
-        return userhome + path[i:]
-else:
-    expanduser = os.path.expanduser
+expanduser = os.path.expanduser
 
 
 def format_permissions(st_mode):
@@ -588,20 +591,16 @@ def copyfile(src, dest):
 
 
 def get_hardlink_function(src, dest):
-    if iswindows:
-        import win32file, win32api
-        colon = b':' if isinstance(dest, bytes) else ':'
-        root = dest[0] + colon
-        try:
-            is_suitable = win32file.GetDriveType(root) not in (win32file.DRIVE_REMOTE, win32file.DRIVE_CDROM)
-            # See https://msdn.microsoft.com/en-us/library/windows/desktop/aa364993(v=vs.85).aspx
-            supports_hard_links = win32api.GetVolumeInformation(root + os.sep)[3] & 0x00400000
-        except Exception:
-            supports_hard_links = is_suitable = False
-        hardlink = windows_fast_hardlink if is_suitable and supports_hard_links and src[0].lower() == dest[0].lower() else None
-    else:
-        hardlink = os.link
-    return hardlink
+    if not iswindows:
+        return os.link
+    from calibre_extensions import winutil
+    if src.startswith(long_path_prefix):
+        src = src[len(long_path_prefix):]
+    if dest.startswith(long_path_prefix):
+        dest = dest[len(long_path_prefix):]
+    root = dest[0] + ':\\'
+    if src[0].lower() == dest[0].lower() and winutil.supports_hardlinks(root):
+        return windows_fast_hardlink
 
 
 def copyfile_using_links(path, dest, dest_is_dir=True, filecopyfunc=copyfile):
@@ -609,6 +608,7 @@ def copyfile_using_links(path, dest, dest_is_dir=True, filecopyfunc=copyfile):
     if dest_is_dir:
         dest = os.path.join(dest, os.path.basename(path))
     hardlink = get_hardlink_function(path, dest)
+    path, dest = make_long_path_useable(path), make_long_path_useable(dest)
     try:
         hardlink(path, dest)
     except Exception:
@@ -622,7 +622,7 @@ def copytree_using_links(path, dest, dest_is_parent=True, filecopyfunc=copyfile)
     hardlink = get_hardlink_function(path, dest)
     try:
         os.makedirs(dest)
-    except EnvironmentError as e:
+    except OSError as e:
         if e.errno != errno.EEXIST:
             raise
     for dirpath, dirnames, filenames in os.walk(path):
@@ -631,7 +631,7 @@ def copytree_using_links(path, dest, dest_is_parent=True, filecopyfunc=copyfile)
         for dname in dirnames:
             try:
                 os.mkdir(os.path.join(dest_base, dname))
-            except EnvironmentError as e:
+            except OSError as e:
                 if e.errno != errno.EEXIST:
                     raise
         for fname in filenames:
@@ -640,3 +640,55 @@ def copytree_using_links(path, dest, dest_is_parent=True, filecopyfunc=copyfile)
                 hardlink(src, df)
             except Exception:
                 filecopyfunc(src, df)
+
+
+rmtree = shutil.rmtree
+
+
+if iswindows:
+    long_path_prefix = '\\\\?\\'
+
+    def make_long_path_useable(path, threshold=200):
+        if len(path) > threshold and os.path.isabs(path) and not path.startswith(long_path_prefix):
+            path = long_path_prefix + os.path.normpath(path)
+        return path
+
+    def is_fat_filesystem(path):
+        try:
+            from calibre_extensions.winutil import filesystem_type_name
+        except ImportError:
+            return False
+        if not path:
+            return False
+        drive = os.path.abspath(path)[0].upper()
+        try:
+            tn = filesystem_type_name(f'{drive}:\\')
+        except OSError:
+            return False
+        # Values I have seen: FAT32, exFAT, NTFS
+        return tn.upper().startswith('FAT')
+
+    def get_long_path_name(path):
+        from calibre_extensions.winutil import get_long_path_name
+        lpath = path
+        if os.path.isabs(lpath) and not lpath.startswith(long_path_prefix):
+            lpath = long_path_prefix + lpath
+        try:
+            return get_long_path_name(lpath)
+        except FileNotFoundError:
+            return path
+        except OSError as e:
+            if e.winerror == 123: # ERR_INVALID_NAME
+                return path
+            raise
+
+else:
+    def make_long_path_useable(path, threshold=200):
+        return path
+
+    def get_long_path_name(path):
+        return path
+
+    def is_fat_filesystem(path):
+        # TODO: Implement for Linux and macOS
+        return False
